;*********************************************************************
; File: crt0.s
;
; Created: 16/11/99, Paul Carpenter
; Modified: 8/6/00, Paul Carpenter - Add stack definitions
;
;               For 79049 LCD Controller board
;
;       HEAVILY modified to remove specifics for a serial example
;
; Description:
; 
;  System initialisation routine - entry point for the application.
;  Power up test routines for ROM and RAM, reports status to 
;  SIO1 (should be 0) using Register subroutine.
;  The stack pointer is initialised, then the hardware initialisation
;  routine called.  The static data areas are then initialised, before
;  the main function is executed.  A simple exit funtion is also
;  supplied.
;
;
;  *** Shadow vector table is proved for HDI-M use.
;   Now made into standard vector table
;
;   Global memory blocks are defined here to ensure placement in RAM 
;   correctly.
;
;   This file MUST be called crt0.o on compilation so that standard
;   library modules can be linked in.
;
; Version History
; V1.00 16/11/99  Paul Carpenter  Original version
; V1.01 24/01/00  Paul Carpenter  Add variables RAM_START
; V1.02 8/6/00    Paul Carpenter  Add stack definition
;*********************************************************************
        .h8300h
	.section .text
	.global _start
	.global _exit
        .global _enable_int
        .global _disable_int

	.extern _hw_initialise
	.extern _main
        .extern _timer_tick
        .extern _rx_int_err0
        .extern _rx_int0
        .extern _tx_int0
        .extern _rx_int_err1
        .extern _rx_int1
        .extern _tx_int1
        .extern _rx_int_err2
        .extern _rx_int2
        .extern _tx_int2
        .extern _check_sum

	.extern _data
	.extern _mdata
	.extern _edata
	.extern _bss
	.extern _ebss
        .extern _stack

        SIO1_SMR = 0xFFFFB8     ; Serial I/O registers (Mode)
        SIO1_BRR = 0xFFFFB9     ; bit rate
        SIO1_SCR = 0xFFFFBA
        SIO1_TDR = 0xFFFFBB     ; Transmit data
        SIO1_SSR = 0xFFFFBC     ; Status
        SIO1_RDR = 0xFFFFBD     ; Receive data
        TCSR     = 0xFFFF8C     ; Timer CSR
        TCNT     = 0xFFFF8D     ; Timer Counter
        RSTCSRR  = 0xFFFF8F     ; Reset CSR (Watchdog)
        RSTCSRW  = 0xFFFF8E     ; Reset CSR for writes
        PADR     = 0xFFFFD9     ; Port A Data Register
        PADDR    = 0xFEE009     ; Port A Data Direction Register
        ;
        ; 3067 Internal RAM address range
        ;
        RAM_START= 0xFFEF20
        RAM_TOP  = 0xFFFF1F
	;
        ; configure SIO for 38400 baud, 8 data, 1 stop & no parity
	;
_start:
	mov.b #0, r0l
        mov.b r0l, @SIO1_SMR:8
	mov.b #0x30, r0l
        mov.b r0l, @SIO1_SCR:8  ; enable RX and TX
        ; currently set to 9600 baud until Hint can match this speed.
        mov.b #64, r0l          ; 51 for 16Mhz or 58 for 18.4MHz or 64 for 20MHz
        mov.b r0l, @SIO1_BRR:8  ; 38400 baud
        ; change to 38,400 baud (15 for count)
        mov.b @SIO1_RDR:8, r0l  ; dummy reads of receive
        mov.b @SIO1_RDR:8, r0l  ; data to clear unknown data
	mov.b #0, r0l
        mov.b r0l, @SIO1_SSR:8  ; clear all flags in serial I/O
        mov.b #0x3F, r0l
        mov.b r0l, @PADDR       ; Set Port A for output of status
        mov.b r0l, @PADR:8      ; Initially set all outputs to 1
	;
	; Check we have not arrived from a Watchdog reset
	;
        xor.b r5l, r5l          ; clear watchdog flag
        mov.b @TCSR:8, r0l        ; read TCSR
	and.b #0x60, r0l        ; WT and TME set??
	beq   reset             ; No must be reset..
        mov.b @RSTCSRR:8, r0l   ; read Watchdog status
        btst  #7, @RSTCSRR:8    ; Watchdog reset bit (WRST) set??
	beq   reset             ; No normal reset.
	mov.b #0xA5, r0h        ; set 'password'
	and.b #0x7F, r0l        ; clear WRST status
        mov.w r0, @RSTCSRW      ; Send to stop Watchdog
	mov.b #'z, r2l          ; Yes must be watchdog
        mov.b r2l, r5l          ; set watchdog reset status to r5l
	bra   reset_1           ; send watchdog restart status
	;
	; send start of RESET char
	;
reset:
	mov.b #'Z, r2l          ; Start of startup and tests
reset_1:
	mov.l #rom_test, er3    ; return address
        jmp   reg_sio           ; goto function to send
	;
	; Perform power up register ONLY tests of ROM/RAM
	;
	; First ROM Checksum (16bit adds to 32 bit result)
rom_test:
	mov.b #'7, r2l          ; report begin ROM test
	mov.l #rom_1, er3       ; return address
        jmp   reg_sio           ; goto function to send
rom_1:
	xor.l er0, er0          ; start with zero checksum
        mov   #_check_sum - 2, er1 ; point to last word to add up
rom_2:
        mov.w @er1, e0          ; fetch next word
	add.w e0, r0            ; 16 bit add of checksum
	bcc   rom_3             ; no overflow
	inc.w #1, e0            ; yes, so inc MS word for 32 bit
rom_3:
	dec.l #2, er1           ; point to next word
	bne   rom_2             ; still more to do
        mov.l @_check_sum, er4  ; get the real checksum
	cmp.l er4, er0          ; compare checksums
;   beq   ram_test              ; fine proceed to next step
;***************************************************************
	bra ram_test      ; debug only ignore CHECKSUM ERROR
;***************************************************************
	;
	; ROM checksum failure is a FATAL error so report it
	;
        jmp   err_sio           ; send character
	;
	; Internal RAM zero and count pattern test 
	; Unless Watchdog or other reasons
	;
ram_test:
	mov.b #'6, r2l          ; Report begin RAM zero test
	mov.l #ram_1, er3       ; return address
        jmp   reg_sio           ; goto function to send
	;
        ; Watchdog and warm start flag checks here
        ;    Check r5l nonzero...then skip RAM tests..
        ;
ram_1:
        cmp.b #0:8, r5l         ; watchdog??
        bne   set_stk           ; yes skip RAM tests..
        mov.l #RAM_TOP + 1, er1 ; point to top of int RAM
	xor.l er0, er0          ; value to send
ram_2:
	mov.l er0, @-er1        ; clear location
	mov.l @er1, er4         ; check actually cleared
	beq   ram_3
        jmp   err_sio
ram_3:
        cmp.l #RAM_START, er1   ; reached bottom of RAM
	bne   ram_2             ; no repeat clear locations test
	;
	; RAM data count pattern test
	;
ram_count:
	mov.b #'5, r2l          ; Report begin RAM count pattern
	mov.l #ram_10, er3      ; return address
        jmp   reg_sio           ; goto function to send
ram_10:
        mov.l #table_1, er4     ; RAM address location table
	mov.b #14, r0l          ; get number of addresses
ram_11:
	mov.l @er4+, er1        ; get next address to write to
	mov.b r0l, @er1         ; Send the decrementing pattern
	dec.b r0l               ; decrement count
	bne   ram_11            ; after sending 1 stop
	mov.l #table_1, er4     ; RAM address location table
	mov.b #14, r0l          ; get number of addresses
ram_12:
	mov.l @er4+, er1        ; get next address to write to
	mov.b @er1, r0h         ; Read the RAM contents
	cmp.b r0l, r0h          ; Same??
        bne   err_sio           ; no report error
	dec.b r0l               ; decrement count to check against
	bne   ram_12            ; after reading 1 stop
	;
	;  At this point it is SAFE to use INTERNAL RAM
	;
        ; initialise the SP for non-vectored code (subroutines)
	;
set_stk:   
        mov.l  #_stack,er7
	;
	; Configure on chip devices
	;
        mov.b #'4, r2l          ; Start of startup status
	mov.l #config, er3      ; return address
	bra   reg_sio           ; goto function to send
	;
	; call the hardware initialiser
	;
config:
	jsr     @_hw_initialise
	;
	; get the boundaries for the .data section initialisation
	;
	mov.l   #_data,er0
	mov.l   #_edata,er1     ; end address initialised data
	mov.l   #_mdata,er2     ; start address initialised data
	cmp.l   er0,er1
	beq     bss_zero        ; Nothing to load so skip
	;
	; loop to load items to initialise RAM with
	;
initload:
	mov.b   @er2+,r3l       ;get from src
	mov.b   r3l,@er0        ;place in dest
	inc.l   #1,er0          ;inc dest
	cmp.l   er0,er1         ;dest == edata?
	bne     initload
	;
	; Loop to clear non-initialised RAM
	; zero out bss
	;
bss_zero:
	mov.l   #_bss,er0       ; get start address
	mov.l   #_ebss,er1      ; and end address
	cmp.l   er0,er1         
	beq     done            ; same skip clear RAM
	sub.b   r2l,r2l         ; clear data to send
bss_loop:
	mov.b   r2l,@er0        ; clear location
	inc.l   #1,er0
	cmp.l   er0,er1
	bne     bss_loop
	;
	; Initialisations complete call the C code modules
	;
done:
	jsr     @_main          ; call the mainline
	;
	; jump to exit
	; reached if we return from main in C code!!!
	; good embedded code should not do this
	;
	mov.l   er0,er4
        bra     _exit


;*********************************************************************
;  Register based SIO routine
;
;  Assumes SIO port has been setup
;  R2L contains byte to send
;  ER3 contains 32 bit address to return to
;
;*********************************************************************
err_sio:
        mov.b #'E, r2l          ; Set up error code to send.
	mov.l #_exit, er3       ; & return address (FATAL)
reg_sio:
        btst  #7, @SIO1_SSR:8   ; wait for ready to send
        beq   reg_sio           ; repeat until bit set
        mov.b r2l, @SIO1_TDR:8  ; send the data byte
        mov.b @SIO1_SSR:8, r2l    ; Get status
        and.b #0x7f, r2l        ; clear TDRE bit to send
        mov.b r2l, @SIO1_SSR:8  ; write back to reg
        jmp   @er3              ; return


;*********************************************************************
;  Dummy exit routine (branch self)
;
;  This should never be reached, with exceptions of 
;  ROM and RAM failures on power up tests
;
;*********************************************************************
_exit:  xor.b   #0x20, r0l
        mov.b   r0l, @PADR:8    ; Set error flag on port A bit 5 (pin 98)
	bra _exit


;*********************************************************************
;  Interupt Enable disable routines
;       really should be a #define inline_asm
;
;  These routines allow C modules to control the Global interupt
;  disable/enable flag in the H8 CCR
;
;       STACK POINTER MUST BE VALID BEFORE CALLING THIS ROUTINE
;       OTHERWISE INTERUPTS AND CALLING THIS FUNCTION WILL CRASH!!
;
;*********************************************************************
_enable_int:
        andc    #0x7f, ccr      ; Enable the CCR interupt flag
        rts

_disable_int:
        orc     #0x80, ccr      ; Disable the CCR interupt flag
        rts


;*********************************************************************
;
;   RAM test Data Count Address Table
;  Only one bit in RAM address range set at any one time..
;
;*********************************************************************
table_1:
        .long  RAM_START        ; Ram start
        .long  0xFFF000
        .long  0xFFF001
        .long  0xFFF002
        .long  0xFFF004
        .long  0xFFF008
        .long  0xFFF010
        .long  0xFFF020
        .long  0xFFF040
        .long  0xFFF080
        .long  0xFFF100
        .long  0xFFF200
        .long  0xFFF400
        .long  0xFFF800
table_end:

;********************************************************************
;
;   Vector table for normal usage
;
;********************************************************************
	.section	.vects
	.global 	_vectable
        .MACRO ADD TO
;EVB use
;        .long   0x5A000000 +  \TO
;target use
        .long \TO
        .ENDM
_vectable:
        ADD     _start          ; Reset     Vec 0       External Pin
        ADD     _exit           ; Reserved  Vec 1
        ADD     _exit           ; Reserved  Vec 2
        ADD     _exit           ; Reserved  Vec 3
        ADD     _exit           ; Reserved  Vec 4
        ADD     _exit           ; Reserved  Vec 5
        ADD     _exit           ; Reserved  Vec 6
        ADD     _exit           ; NMI       Vec 7       External pin
        ADD     _exit           ; TRAPA 0   Vec 8       Software
        ADD     _exit           ; TRAPA 1   Vec 9
        ADD     _exit           ; TRAPA 2   Vec 10
        ADD     _exit           ; TRAPA 3   Vec 11
        ADD     _exit           ; IRQ 0     Vec 12      External pins
        ADD     _exit           ; IRQ 1     Vec 13
        ADD     _exit           ; IRQ 2     Vec 14
        ADD     _exit           ; IRQ 3     Vec 15
        ADD     _exit           ; IRQ 4     Vec 16
        ADD     _exit           ; IRQ 5     Vec 17
        ADD     _exit           ; Reserved  Vec 18
        ADD     _exit           ; Reserved  Vec 19
        ADD     _exit           ; Watchdog  Vec 20      Watchdog
        ADD     _exit           ; CMI       Vec 21      Refresh
        ADD     _exit           ; Reserved  Vec 22
        ADD     _exit           ; ADI (END) Vec 23      A/D
        ADD     _timer_tick     ; ITU0-IMIA Vec 24      ITU-0
        ADD     _exit           ; ITU0-IMIB Vec 25
        ADD     _exit           ; ITU0-OVF  Vec 26
        ADD     _exit           ; Reserved  Vec 27
        ADD     _exit           ; ITU1-IMIA Vec 28      ITU-1
        ADD     _exit           ; ITU1-IMIB Vec 29
        ADD     _exit           ; ITU1-OVF  Vec 30
        ADD     _exit           ; Reserved  Vec 31
        ADD     _exit           ; ITU2-IMIA Vec 32      ITU-2
        ADD     _exit           ; ITU2-IMIB Vec 33
        ADD     _exit           ; ITU2-OVF  Vec 34
        ADD     _exit           ; Reserved  Vec 35
        ADD     _exit           ; ITU3-CMIA0       Vec 36      ITU-3 8 bit
        ADD     _exit           ; ITU3-CMIB0       Vec 37
        ADD     _exit           ; ITU3-CMIA1/CMIB1 Vec 38
        ADD     _exit           ; TOVI0/TOVI1      Vec 39
        ADD     _exit           ; ITU4-CMIA2       Vec 40      ITU-4 8 bit
        ADD     _exit           ; ITU4-CMIB3       Vec 41
        ADD     _exit           ; ITU4-CMIA1/CMIB1 Vec 42
        ADD     _exit           ; TOVI2/TOVI3      Vec 43
        ADD     _exit           ; DEND0A    Vec 44      DMAC
        ADD     _exit           ; DEND0B    Vec 45
        ADD     _exit           ; DEND1A    Vec 46
        ADD     _exit           ; DEND1B    Vec 47
        ADD     _exit           ; Reserved  Vec 48
        ADD     _exit           ; Reserved  Vec 49
        ADD     _exit           ; Reserved  Vec 50
        ADD     _exit           ; Reserved  Vec 51
        ADD     _rx_int_err0    ; ERI0      Vec 52      SCI-0
        ADD     _rx_int0        ; RXI0      Vec 53
        ADD     _tx_int0        ; TXI0      Vec 54
        ADD     _exit           ; TEI0      Vec 55
        ADD     _rx_int_err1    ; ERI1      Vec 56      SCI-1
        ADD     _rx_int1        ; RXI1      Vec 57
        ADD     _tx_int1        ; TXI1      Vec 58
        ADD     _exit           ; TEI1      Vec 59
        ADD     _rx_int_err2    ; ERI2      Vec 60      SCI-2
        ADD     _rx_int2        ; RXI2      Vec 61
        ADD     _tx_int2        ; TXI2      Vec 62
        ADD     _exit           ; TEI2      Vec 63

;********************************************************************
;
;   temp_array for building binary I/O blocks of data for sending to
;   host RAM or Flash ROM, also used when building blocks of data and
;   commands to send to the Filter and LUT Loaders.
;
;********************************************************************
	.section	.array
	.global 	_temp_array
        .global         _temp_array1
_temp_array:                    ;place holder for temp_array (256 bytes long)
                . = . + 256
_temp_array1:                   ;place holder for temp_array (256 bytes long)


;********************************************************************
;
;   Buffers area of RAM for Serial I/O
;
;********************************************************************
        .section        .buffs
        .global         _rx_buf0
        .global         _tx_buf0
        .global         _rx_buf1
        .global         _tx_buf1
        .global         _rx_buf2
        .global         _tx_buf2

_rx_buf0:                       ; SIO 0 buffers
        . = . + 32
_tx_buf0:
        . = . + 64
_rx_buf1:                       ; SIO 1 buffers
        . = . + 32
_tx_buf1:
        . = . + 64
_rx_buf2:                       ; SIO 2 buffers
        . = . + 32
_tx_buf2:
        . = . + 64

