/**************************************************************
h8core_b.c                  79049 LCD Controller Board

    H8 core function for controlling Digital LCD controller.
    (Upgrade from 79036 LCD controller software).

    Date    5/06/2000

    Author  Paul Carpenter

    This module is the 'main' function for all the C modules
    used and controlling loop.

    Module crt0.s is the startup core to initialise for C

Version History

Rev     Date        Author          Changes
Special example version....

***************************************************************/
#define H8300H  1
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include "h83067f.h"                /* register definitions             */
#include "codes.h"                  /* defines of error/status codes.   */
#include "serial.h"                 /* SIO functions                    */
#include "console.h"                /* console port 1 char I/O          */
#include "parse.h"                  /* Command parser definitions       */

/********* CUSTOM crt0 address definitions of special memory locations **/
extern unsigned char temp_array[ ];
extern unsigned char temp_array1[ ];

unsigned long check_sum __attribute__ ((section (".checks"))) = 0xFFFFFFFF;

char company[] __attribute__ ((section (".strings"))) =
        "\n\rPC Services, United Kingdom \n\r";
char date[] __attribute__ ((section (".strings"))) =
        "17/07/2000";
char product[] __attribute__ ((section (".strings"))) =
        "EXAMPLE of interupt SIO code.\n\r";
char rev[] __attribute__ ((section (".strings"))) =
        "00.00.14";
char status_str[] __attribute__ ((section (".strings"))) =
        "\n\r%c%02X>";
char signon_str[] __attribute__ ((section (".strings"))) =
        "%s%sRev %s\t Date %s";
char version_str[] __attribute__ ((section (".strings"))) =
        "\n\r%s\t%s";

/**************** Global Variables  *********************************/
                                    /********************************/
BYTE    status;                     /* command status see codes.h   */
                                    /********************************/
BYTE    sys_status;                 /* system status see codes.h    */
                                    /********************************/
BYTE    mode;                       /* System mode                  */
                                    /* 0 = 'C' code start           */
                                    /* 0x20 = Normal running        */
                                    /* 0x40 = Setup mode            */
                                    /* 0x80 = Flash programming     */
                                    /********************************/
BYTE    time_count;                 /* Current timer ticks cyclic   */
BYTE    old_time;                   /* previous number of ticks     */
                                    /********************************/
BYTE    event_ticks[ MAX_EVENTS ][ 2 ];  /* table of ticks compare  */
                                    /* values to match up against   */
                                    /* time_count for when to action*/
                                    /* respective function in       */
                                    /* (*funcs)[]                   */
                                    /* Second value is 1 to enable  */
                                    /********************************/
int     i, j;

/****************** Prototypes *********************************/
int     main( void );
void    timer_tick( void );
void    command_check( void );
int     send_status( void );
void    send_signon( void );
int     send_version( void );

/* crt0 special functions */
extern void disable_int( void );    /* Disable H8 Interupts */
extern void enable_int( void );     /* Enable H8 Interupts */

/* table of event processes */
void (*funcs[ MAX_EVENTS ])(void) __attribute__ ((section (".strings"))) =
            {                       /* Order must match event_ticks[]*/
            NULL,
            NULL,
            NULL,
            NULL,
            command_check           /* Command parsing check    */
            };                      /****************************/

/******************** Code starts here *************************/
int main( void )
{
serial_init( -1 );                      /* Init Serial ports                */
status = 0;                             /* Still in power up tests          */
mode = 0;
sys_status = SW_STEREO;                 /* start in stereo..    */

enable_int( );                          /* Be sure interupts enabled        */
putchar( 1, '3' );                      /* Status 3 External Flash ROM test */
                                        /* Checksum all external flash      */

/* Status 2 is ext (to CPU) RAM zero test           */
putchar( 1, '2' );                   /* Power up tests not written yet   */
/* Status 1 is ext RAM count pattern test           */
putchar( 1, '1' );                   /* Power up tests not written yet   */

status = STAT_OK;
error = 0;

send_signon( );                     /* Display sign on version etc. */

ITU_TSTR = 0x01;                    /* Start the timer int counting */
old_time = time_count;

/* disable all timer events Only allow command processing   */
for( loop = 0; loop < MAX_EVENTS; loop++ )
   event_ticks[ loop ][ 1 ] = 0;

event_ticks[ HEAD ][ 0 ] = time_count + 2;/* setup events to process  */
event_ticks[ HEAD ][ 1 ] = 1;           /* enable the tick event    */
event_ticks[ COMMS ][ 0 ] = 20;         /* check 200ms for a command*/
event_ticks[ COMMS ][ 1 ] = 1;          /* Enable it                */
mode = NORMAL;                          /* In normal running        */

/******************** Main processing loop **************************/
/* This loop is continuously checking timer counts for what to      */
/* action, when the global counter has changed from last value the  */
/* table of actions is scanned to find matches of functions to      */
/* process at this time count..                                     */
/*                                                                  */
/*  Highest priority is headtracker comms (if detected)             */
/*  Then Backlight control onwards...                               */
/*  Lowest priority is debug port command parsing                   */
/*                                                                  */
/*  The only time this loops is not running is during large binary  */
/*  up/downloads on the monitoring port, or during flash            */
/*  programming time. This is due to exclusive nature of CPU that   */
/*  is required.                                                    */
/********************************************************************/
for( ;; )
   {
   PADR |= 16;
   while( time_count == old_time );     /* wait for next time tick  */
   PADR &= ~16;
   old_time = time_count;               /* save current value       */
   PADR |= 0x01;                        /* Set port A.0 start checks*/
   for( loop = 0; loop < MAX_EVENTS; loop++ )
      if( event_ticks[ loop ][ 1 ] == 1 )   /* Only if enabled      */
        if( event_ticks[ loop ][ 0 ] == old_time ) /* check due for action??  */
          (*funcs[ loop ])();           /* Yes call event from table*/
   PADR &= 0xFE;
   }
}

/********************************************/
/*  timer_tick - interupt function          */
/*                                          */
/*  Increments time_count every timer reset */
/*  on match interupt                       */
/********************************************/
#pragma interrupt
void timer_tick( void )
{
ITU_TISRA &= 0xFE;                      /* Clear down interupt status */
time_count++;                           /* to release IRQ           */
}


/********************************************************************/
/* Check for commands on debug and expansion (if enabled) ports     */
/********************************************************************/
void command_check( void )
{
PADR |= 0x02;                           /* set PA.1 command proc scope */
/* Check for messages on Serial 1 (monitor/debug)   */
if( SIO_status[ 1 ][ 0 ] & 0x80 )       /* CR - Full command received */
  {
  if( ( parse_command( ) ) > 0 );       /* Process the command line */
    {
    /************************************************/
    /* At this point we have a series of commands   */
    /* Now we need to check them against list of    */
    /* commands for validity..                      */
    /************************************************/
    error = -1;
    for( loop = 0; loop < NO_COMMANDS; loop++ )
       if( ( strcmp( commands[ loop ].token, paramv[ 0 ] ) ) == 0 )
         break;
    if( loop == NO_COMMANDS )
      error = 0;                        /* Invalid command          */
    else
      {                                 /* Check min and max args   */
      if( paramc < commands[ loop ].min_arg )
        {                               /* work out missing parameter */
        error = commands[ loop ].min_arg -
                        ( commands[ loop ].min_arg - paramc );
        error = ( error > 3 ) ? 3 : error;
        }
      if( paramc > commands[ loop ].max_arg )
        error = PARAM_OVERFLOW;                 /* too many parameters  */
      if( !( mode & commands[ loop ].modes ) )  /* wrong mode           */
        error = BAD_MODE;
      }
    if( error >= 0 )
      param_error( error );
    else
      {
      error = 0;                        /* start with no errors     */
      loop = commands[ loop ].func( );  /* Action the command       */

      /****************************************/
      /* Command return values signify type of*/
      /* error condition, 'error' contains the*/
      /* error number.. return values are     */
      /* -2 Hardware failure                  */
      /* -1 Parameter error                   */
      /*  0 Success                           */
      /*  1 Success already printed status    */
      /*  2 Setup mode success do not print   */
      /*          status again                */
      /****************************************/
      if( loop < 0 )                    /* command errored          */
        param_error( error );           /* report error             */
      else
        if( loop == 0 )                 /* no error                 */
          {                             /* so send status           */
          switch( mode )
                {
                case NORMAL:    status = STAT_OK;
                                break;
                case SETUP:     status = SETUP_OK;
                                break;
                case FLASH:     status = FLASH_OK;
                                break;
                }
          send_status( );
          }
      }
    clear_command( );                   /* clear command from buffer */
    }
  }
event_ticks[ COMMS ][ 0 ] = time_count + 20; /* check 200ms for command */
PADR &= 0xFD;                           /* Clear command proc scope */
}


/********************************************/
/* Send_Status or error condition           */
/*            Sends Status sequence to SIO1 */
/*                                          */
/*   Status < 128 is an actual status       */
/*                                          */
/*   Status > 127 is an actual error code   */
/*                                          */
/*  Returns 1 to indicate valid status sent */
/*                                          */
/********************************************/
int send_status( void )
{
mon_printf( status_str, ( status > 127 ) ? 'E':'S', status );
return( 1 );                            /* return status printed no errors */
}


/********************************************/
/*  Send_Signon                             */
/*            Sends the Company name        */
/*                      Product name        */
/*                      Version number      */
/*                      Version date        */
/********************************************/
void send_signon( void )
{
/* Send Sign on message to show after memory power-up testing */
mon_printf( signon_str, company, product , rev, date );
}


/********************************************/
/*  Send_Version                            */
/*            Sends the Version number      */
/*                      Version date        */
/********************************************/
int send_version( void )
{
mon_printf( version_str, rev, date );
return( 0 );
}
