/**************************************************************
parse.c                                     (79049)

    H8 Debug/Monitor port Command processing

    Date    7/12/99

    Author  Paul Carpenter

    Module to perform command parsing and basic error checks
    for command is valid, and at least the number of arguments
    to match min and max requirements.

    No argument validity checks are performed.

Version History

Rev     Date    Author          Changes
1.00    17/11/99 Paul Carpenter  Original Version (79049 from 79036)
***************************************************************/
#include <string.h>

#include "codes.h"
#include "h8core_b.h"
#include "console.h"
#include "setup.h"
#include "flash.h"
#include "serial.h"

extern int     start( void );       /* restart address (crt0.s) */

/****************** Prototypes *********************************/
void    param_error( unsigned int );
int     conv_byte( int, BYTE, BYTE, BYTE );
int     conv_word( int, int, int, int );
int     conv_address( int );
char    *conv_number( char *, unsigned long * );
int     parse_command( void );
void    clear_command( void );
int     send_help( void );
int     comm_byte( int );
int     comm_bin( int );
int     comm_common( int, int );

/**************** Global Variables  *****************************/
/* The following variables are GLOBAL for COMMAND               */
/* processing only as ONLY one command can be active at a time  */
BYTE paramc;                    /* Number parameters & command strings  */
                                /* found in last command line received  */
char params[ RX_SIZE + 3 ];     /* Parameters buffer                    */
int error;                      /* error code for command processing    */
int loop;                       /* counter for parameter processing     */
char *paramv[ MAX_PARAM ];      /* pointers substrings in command line  */

union temp_val                  /* temp storage for up to 6 parameters  */
        {
        unsigned char   ch;
        short         word;
        } temp_numb[ MAX_PARAM - 1 ];
unsigned long number;           /* temp storage for number conversions  */
unsigned long temp_addr[ 3 ];   /* temp storage decoded HEX addresses   */
                                /* 0 = source start address             */
                                /* 1 = source end address               */
                                /* 2 = destination start address        */

const struct  comm_table
    {
    char            *token;         /* Command names as received UPPER  */
    unsigned char   min_arg;        /* Min allowable args (1=command )  */
    unsigned char   max_arg;        /* Max allowable args               */
    unsigned char   modes;          /* modes allowed (bit combination)  */
                                    /* 0 = not supported                */
                                    /* 1 = Normal                       */
                                    /* 2 = Debug                        */
                                    /* 4 = Flash                        */
    int     ( *func )( void );      /* function address                 */
    } commands[ NO_COMMANDS ] =
        {
        { "+",          1, 1, NORMAL+SETUP+FLASH,   test_serial },
        { "?",          1, 1, NORMAL+SETUP+FLASH,   send_status },
        { "V",          1, 1, NORMAL+SETUP+FLASH,   send_version },
        { "H",          1, 1, NORMAL+SETUP+FLASH,   send_help },
        { "SETUP",      1, 1, NORMAL,               comm_setup },
        { "RESTART",    1, 1, NORMAL+SETUP,         start },
        { "BAUD",       2, 2, SETUP,                comm_baud },
        { "READ",       2, 3, SETUP,                read_mem },
        { "WRITE",      2, 3, SETUP,                write_mem },
        { "CS",         1, 2, SETUP,                comm_checksum },
        { "EXIT",       1, 1, SETUP,                ret_normal },
        { "QUIT",       1, 1, SETUP,                ret_normal },
        { "NORMAL",     1, 1, SETUP,                ret_normal }
        };

char help_mess[] __attribute__ ((section (".strings"))) =
            "\n\rAll modes\n\r"
            "?\t\t- Repeat last status\tV\t- Display software version\n\r"
            "RESTART\t- Restart processor\tH\t- This help message\n\r"
            "U\t\t- Get User Panel Readings\n\r";
char help_norm[] __attribute__ ((section (".strings"))) =
            "\n\rNormal Mode\n\r"
            "SETUP\t\t- Enter Setup Mode";

char help_setup[] __attribute__ ((section (".strings"))) =
            "\n\rSetup Mode\n\r"
            "BAUD dddd\t\t\t- Change Baud Rate\n\r"
            "READ ssssss [eeeeee]\t\t- Read memory block from ssssss [to eeeeee]\n\r"
            "WRITE ssssss [eeeeee]\t\t- Write memory block from ssssss [to eeeeee]\n\r"
            "CS xxxxxx\t\t-Checksum of flash memory"
            "EXIT or QUIT or NORMAL\t\t- Return to Normal mode";

                
/********************************************************/
/* Parse_command convert command line into a series of  */
/*                  pointers to arguments and count     */
/*                                                      */
/*  Input parameter NONE                                */
/*                                                      */
/*  Assumptions     getline function converts HT to SP  */
/*                                                      */
/*  Error conditions  C7 if decoding of arguments       */
/*                      exceeds limits of MAX_PARAM     */
/*                                                      */
/*  Outputs        *paramv[ n ] filled with pointers to */
/*                          parameters                  */
/*                  paramc contains the parameter count */
/*                                                      */
/* Description                                          */
/*      Converts a command line into a parameter count  */
/*  (paramc) and a table of parameter string pointers   */
/*  ( *paramv[] ). By determining spaces as end of      */
/*  parameters, whilst '\0', '\r', '\n' are terminators */
/*  of the command line.                                */
/********************************************************/
int parse_command( void )
{
unsigned char   *ptr;

/* Scan line until terminator, parameter found save pointer and inc  count */
/* of parameters    */
ptr = params;
getline( 1, ptr );
while( *ptr != '\0' && *ptr != '\r' && *ptr != '\n' )
  {
  if( *ptr == ' ' )                 /* White space between parameters ? */
    {
    *ptr++ = '\0';                  /* terminate last parameter */
    while( *ptr == ' ' && *ptr != '\0' 
            && *ptr != '\r' && *ptr != '\n' )
         *ptr++;                    /* skip white spacing */
    }
  else                              /* actually a parameter */
    {
    if( paramc > MAX_PARAM )        /* too many parameters ? */
      {
      error = 7;
      param_error( error );
      clear_command( );             /* flush the command as invalid */
      return( -1 );
      }
    paramv[ paramc ] = ptr;         /* save position as we have */
    paramc++;                       /* found another parameter */
    while( *ptr != ' ' && *ptr != '\0'
            && *ptr != '\r' && *ptr != '\n' )
         *ptr++;                    /* skip remaining chars in parameter */
    }
  }
putchar( 1, ' ' );                  /* space out command and reply */
*ptr = '\0';                        /* Terminate string */
return( paramc );
}


/********************************************************/
/* clear_command resets command present flags & counts  */
/*                                                      */
/*  Input parameter NONE                                */
/*                                                      */
/*  Error conditions  NONE                              */
/*                                                      */
/*  Outputs         paramc zeroed                       */
/*                  *paramv[ n ] set to NULL            */
/*                  rx_buf1 cleared to '\0'             */
/*                  All this indicates nothing entered  */
/********************************************************/
void clear_command( void )
{
/* function to free up memory buffer after command processing */
paramc = 0;
for( loop = 0; loop < MAX_PARAM; loop++ )
   paramv[ loop ] = NULL;
for( loop = 0; loop < RX_SIZE + 3; loop++ )
   params[ loop ] = '\0';
}


/********************************************************/
/* conv_byte convert a (hex) parameter to a byte value  */
/*                                                      */
/*  Input parameter int n - Range 1 to F                */
/*                          which parameter to convert  */
/*                  int min - minimum value for range   */
/*                  int max - maximum value for range   */
/*                  int mask - BIT mask value for range */
/*                                                      */
/*  Error conditions  If n out of range function        */
/*                          returns with NO action      */
/*                      error set to 7 for C7 error     */
/*                  Parameter inavlid if conversion is  */
/*                  not within constraints of min, max  */
/*                  and mask.                           */
/*                                                      */
/*  Return value    0 for success or parameter in error */
/*                      value usually n as passed in.   */
/*                                                      */
/*  Outputs         temp_numb.ch[ n - 1 ] is updated to */
/*                      hold a valid conversion         */
/*                                                      */
/*  Error codes     Output to users on SIO1             */
/*                  C4   Parameter 1 INVALID            */
/*                  C5   Parameter 2 INVALID            */
/*                  C6   Parameter 3 (or more) INVALID  */
/*                  C7   Too many parameters            */
/********************************************************/
int conv_byte( int n, BYTE min, BYTE max, BYTE mask )
{
if( n > MAX_PARAM || n == 0 )
  {
  error = 7;
  return( n );
  }
conv_number( paramv[ n ], &number );
if( number > max || number < min || ( number & ~mask ) )
  {
  error = ( error > 2 ) ? 3 : 2;
  return( n );
  }
temp_numb[ n - 1 ].ch = (unsigned char)number;
return( 0 );
}


/********************************************************/
/* conv_word convert a (hex) parameter to a word value  */
/*                                                      */
/*  Input parameter int n - Range 0 to F                */
/*                          which parameter to convert  */
/*                  int min - minimum value for range   */
/*                  int max - maximum value for range   */
/*                  int mask - BIT mask value for range */
/*                                                      */
/*  Error conditions  If n out of range function        */
/*                          returns with NO action      */
/*                      error set to 7 for C7 error     */
/*                                                      */
/*  Return value    0 for success or parameter in error */
/*                      value usually n as passed in.   */
/*                                                      */
/*  Outputs         temp_numb.ch[ n - 1 ] is updated to */
/*                      hold a valid conversion         */
/*                                                      */
/*  Error codes     Output to users on SIO1             */
/*                  C4   Parameter 1 INVALID            */
/*                  C5   Parameter 2 INVALID            */
/*                  C6   Parameter 3 (or more) INVALID  */
/*                  C7   Too many parameters            */
/********************************************************/
int conv_word( int n, int min, int max, int mask )
{
if( n > MAX_PARAM )
  {
  error = 7;
  return( n );
  }
conv_number( paramv[ n ], &number );
if( number > max || number < min || ( number & ~mask ) )
  {
  error = ( error > 2 ) ? 3 : 2;
  return( n );
  }
temp_numb[ n - 1 ].word = (unsigned short)number;
return( 0 );
}


/********************************************************/
/* conv_address convert a (hex) parameter to an address */
/*                                                      */
/*  Input parameter int n - Range 2 to 4                */
/*                          which parameter to convert  */
/*                                                      */
/*  Error conditions  If n out of range function        */
/*                          returns with NO action      */
/*                      error set to 7 for C7 error     */
/*                    Address greater than 24 bits      */
/*                                                      */
/*  Return value    0 for success or parameter in error */
/*                      value usually n as passed in.   */
/*                                                      */
/*  Outputs         temp_address[ n - 1 ] is updated to */
/*                      hold a valid conversion         */
/*                                                      */
/*  Error codes     Output to users on SIO1             */
/*                  C4   Parameter 1 INVALID            */
/*                  C5   Parameter 2 INVALID            */
/*                  C6   Parameter 3 (or more) INVALID  */
/*                  C7   Too many parameters            */
/*                                                      */
/* NOTE     NO address checks are performed             */
/********************************************************/
int conv_address( int n )
{
if( n > MAX_PARAM )
  {
  error = 7;
  return( n );
  }
conv_number( paramv[ n ], &number );
if( number > (unsigned long)0xFFFFFF )
  {
  error = ( error > 2 ) ? 3 : 2;
  return( n );
  }
temp_addr[ n - 1 ] = number;
return( 0 );
}


/********************************************************/
/* conv_number   Convert a string to a number using a   */
/*                                                      */
/*  Input parameter char *p pointer to string to convert*/
/*                  unsigned long *dst pointer to result*/
/*                                                      */
/*  Error conditions  If type out of range              */
/*                    function returns with result=zero */
/********************************************************/
char *conv_number( char *p, unsigned long *dst )
{
char c;

*dst = 0;
while( *p )
    {
    c = *p++;
    if( c >= '0' && c <= '9' )
      c = c - '0';
    else
      if( c >= 'A' && c <= 'F' )
        c = c - 'A' + 10;
    else
      break;
    *dst = *dst << 4;
    *dst += ( unsigned long )c;
    }
return p;
}


/********************************************************/
/* Param_error Parameter error reporting                */
/*            Sends error message to SIO1               */
/*                                                      */
/*  Input parameter unsigned int type - Range 0 to 7    */
/*                  Added to 0xC0 base error            */
/*                                                      */
/*  Error conditions  If type out of range              */
/*                    function returns with NO action   */
/*                                                      */
/*  Outputs        Error codes output to users on SIO1  */
/*                  C0   Unrecognised Command           */
/*                  C1   Parameter 1 missing            */
/*                  C2   Parameter 2 missing            */
/*                  C3   Parameter 3 (or more) missing  */
/*                  C4   Parameter 1 INVALID            */
/*                  C5   Parameter 2 INVALID            */
/*                  C6   Parameter 3 (or more) INVALID  */
/*                  C7   Too many parameters            */
/*                  CD   Wrong mode for command         */
/*                  CF   Hardware error in actioning    */
/********************************************************/
void param_error( unsigned int type )
{
if( type < HARDWARE_FAIL )
  status = BAD_COMMAND + type;    /* Create full status value */
else
  status = type;
send_status( );
}


/********************************************************/
/* Send_help  display simple help message               */
/*            Sends help message to SIO1                */
/*                                                      */
/*  Input parameter         NONE                        */
/*                                                      */
/*  Error conditions        NONE                        */
/*                                                      */
/*  Outputs                 SIO1 has the ASCII strings  */
/*                          output for the help message */
/********************************************************/
int send_help( void )
{
putstring( 1, help_mess );
if( mode == NORMAL )
  putstring( 1, help_norm );
else
  putstring( 1, help_setup  );
return( 0 );
}


/********************************************************/
/* comm_byte  Convert several bytes in range 0-FF       */
/*                                                      */
/*  Input parameter         Number of parameters        */
/*                                                      */
/*  Error conditions        Conversion error            */
/*                                                      */
/*  Outputs                 temp_numb[n].ch filled      */
/********************************************************/
int comm_byte( int numb )
{
return( comm_common( numb, 0 ) );
}


/********************************************************/
/* comm_bin  Convert several bytes in range 0-1 (binary)*/
/*                                                      */
/*  Input parameter         Number of parameters        */
/*                                                      */
/*  Error conditions        Conversion error            */
/*                                                      */
/*  Outputs                 temp_numb[n].ch filled      */
/********************************************************/
int comm_bin( int numb )
{
return( comm_common( numb, 1 ) );
}


int comm_common( int numb, int mode )
{
BYTE    rep;

for( rep = 1; rep <= numb; rep++ )
   if( mode == 0 )
     {
     if( ( error = conv_byte( rep, IOUT_MIN, IOUT_MAX, IOUT_MAX ) ) != 0 )
       break;
     }
   else
     if( mode == 1 )
       if( ( error = conv_byte( rep, OFF, ON, ON ) ) != 0 )
         break;
if( error != 0 )
  {
  if( error > 3 )                   /* convert error code */
    error = 3;
  error += BAD_PARAM1;              /* add offset for invalid type */
  return( -1 );
  }
return( 0 );
}
