/**************************************************************
FLASH.C            FLASH programming            (79049)

    H8 Debug/Monitor port Command processing

    Date    8/5/00

    Author  Paul Carpenter

    Module to perform command processing for changing or reading
    the FLASH memory on Digital LCD controller. 

    Argument validity checks are performed.
    Processing of data for sending or receiving is also performed

    *** All comm_xxxx functions require use of the global variables
        paramc      count of parameters
        *paramv[]   array of pointers to parameters

    These are set by the function parse_comm() in ser_int.c
***************************************************************/
#include    <stdarg.h>
#include    <string.h>
#include    "h83067f.h"
#include    "ascii.h"
#include    "codes.h"
#include    "h8core_b.h"
#include    "setup.h"
#include    "parse.h"
#include    "serial.h"
#include    "console.h"


/*********************** Prototypes ***************************/
unsigned long int calc_checksum( unsigned int *, unsigned int * );
int comm_checksum( void );
int send_hex_block( BYTE *, unsigned long, int );
int get_hex_packet( BYTE *, int );

unsigned long   checksum;

char calc_check_str[] __attribute__ ((section (".strings"))) =
    "\n\rRecalculating Checksum";
char rep_check_str[] __attribute__ ((section (".strings"))) =
    "\n\rChecksum   Int = %08lX";
char rep_calc_str[] __attribute__ ((section (".strings"))) =
    "\n\rCalculated Int = %08lX";
char h8_check_fail[] __attribute__ ((section (".strings"))) =
    "\n\rH8 Flash Checksum failure";


/********************************************************/
/* calc_checksum- Command processing calculate Checksum */
/*                                                      */
/*  Implicit Input Parameters   NONE                    */
/*                                                      */
/*  Input parameter integer pointers to start address   */
/*                  and first integer address AFTER the */
/*                  last address read                   */
/*                                                      */
/*  Error conditions  Invalid or insufficient parameter */
/*                    return value of ZERO. Which could */
/*                    also be a valid checksum, be      */
/*                    careful                           */
/*                                                      */
/*  Outputs         NONE                                */
/*                                                      */
/*  Error codes     NONE                                */
/*                                                      */
/*  Return Values   0 - Invalid parameter or error      */
/*                  Any other value is a calculated     */
/*                  checksum                            */
/*                                                      */
/*  Description                                         */
/*      First check that end address is larger than     */
/*      start address, and that both addresses are int  */
/*      aligned. Any of these error conditions returns  */
/*      value of ZERO.                                  */
/*                                                      */
/*      Next phase is to add up 16 bit values in the    */
/*      area of MEMORY (no check that it is a Flash ROM */
/*      address), into a 32bit result.                  */
/*                                                      */
/*      At end of block return 32 bit result            */
/********************************************************/
unsigned long int calc_checksum( unsigned int *start, unsigned int *b_end )
{
unsigned long int ret_val;

if( start > b_end || ( (unsigned long)start & 0x01 )
                || ( (unsigned long)b_end & 0x01 ) )
  return( 0 );
ret_val = 0;
while( start < b_end )
    ret_val += ( unsigned long )( *start++ );
if( ret_val == 0 )
  ret_val++;                            /* zero returned would be error!! */
return( ret_val );                      /* All completed */
}


/********************************************************/
/* comm_checksum- Command processing report Checksum H8 */
/*                                                      */
/*  Implicit Input Parameters                           */
/*                      ASSUMES that 32 bit checksum    */
/*                      is the last four bytes of ROM   */
/*                                                      */
/*  Input parameter     NONE                            */
/*                                                      */
/*  Error conditions  At this level checksum comparison */
/*                                                      */
/*  Outputs (SIO1) Error codes output to users on SIO1  */
/*                                                      */
/*                  C0  Unrecognised Command            */
/*                  C7  Too many parameters             */
/*                  CF  Hardware failure  on comparison */
/*                                                      */
/* Return Values    0 - Success                         */
/*                 -1 - Parameter error                 */
/*                 -2 - Hardware error (comparison fail)*/
/*                                                      */
/* Description                                          */
/*      If no command line parameters supplied then     */
/*      report actual Checksum in Flash (H8 & ext)      */
/*      Calculate a checksum and sends that to the host */
/*                                                      */
/*      If a comparison fails the checksum is followed  */
/*      by error CF, otherwise status normal is sent    */
/********************************************************/
int comm_checksum( void )
{
if( paramc == 2 )
  {
  /* paramv[ 1 ] is the date string to store for last time checksum updated */
  if( strlen( paramv[ 1 ] ) > 20 )
    paramv[ 1 ][ 20 ] = '\0';               /* Ensure string is 20 chars max */
  /* calc ext checksum and write it to FLASH_CHECK */
  putstring( 1, calc_check_str );
  }
else
  {
  mon_printf( rep_check_str, ( *( unsigned long *)check_sum ) );

  /* go add up 16 bit values into 32 bit result */
  checksum = calc_checksum( ( unsigned int *)0x0, ( unsigned int *)check_sum );
  mon_printf( rep_calc_str, checksum );
  /* compare the value against the stored value */
  if( checksum != *(unsigned long *)check_sum && checksum != 0 )
    putstring( 1, h8_check_fail );

  if( error )
    {
    error = HARDWARE_FAIL;
    return( -2 );
    }
  }
return( 0 );                            /* All completed */
}


/********************************************************/
/* send_hex_block - Send ASCII Hex data blocks          */
/*                                                      */
/*  Implicit Input Parameters   NONE                    */
/*                                                      */
/*  Input parameter         BYTE ptr to start of block  */
/*                          unsigned long count bytes   */
/*                          int  port to send ASCII Hex */
/*                                                      */
/*  Error conditions        Invalid parameters          */
/*                                                      */
/*  Outputs                 NONE                        */
/*                                                      */
/* Return Values    0 - Success                         */
/*                 -1 - Parameter error                 */
/*                                                      */
/* Description                                          */
/*  Sends block of data of 'size' bytes from address    */
/*  'start' to SCI 'port'. Sent as blocks of data in    */
/*  ASCII hex format of 16 bytes maximum per line       */
/*  terminated in CR/LF                                 */
/********************************************************/
int send_hex_block( BYTE *start, unsigned long size, int port )
{
BYTE    count;
BYTE    temp_str[ 3 ];
    
if( port < 1 || port > 2 )
  return( -1 );

if( ( start + size ) > ( &RAM_TOP + 1 ) )
  return( -1 );

count = 0;
putstring( port, crlf );                  /* start by syncing data */
while( size-- > 0 )
     {
     temp_str[ 2 ] = '\0';
     GenHexNumber( temp_str, (unsigned long)*start++, 2, '0' );
     putstring( port, temp_str );
     if( count++ == 15 )
       {
       putstring( port, crlf );
       count = 0;
       }
     else
       putchar( port, ' ' );
     }
if( count > 0 )
  putstring( port, crlf );
return( 0 );
}


/********************************************************/
/* get_hex_packet  - Receive Hex data packet            */
/*                                                      */
/*  Implicit Input Parameters   NONE                    */
/*                                                      */
/*  Input parameter         BYTE ptr where to save      */
/*                                  hex data packet     */
/*                          int  port to get ASCII Hex  */
/*                                                      */
/*  Error conditions        Invalid parameters          */
/*                                                      */
/*  Outputs                 Decoded bytes received      */
/*                                                      */
/* Return Values    0 - Success                         */
/*                 -1 - Parameter error                 */
/*                 -2 - comms error                     */
/*                 -3 - Non Ascii hex character         */
/*                                                      */
/* Description                                          */
/*  Receives block of data of bytes from SCI 'port'.    */
/*  Received as blocks of data in ASCII hex format of   */
/*  16 bytes maximum per line terminated in CR/LF       */
/********************************************************/
int get_hex_packet( BYTE *dest, int port )
{
int     count;
    
if( dest > &RAM_TOP )
  return( -1 );

while( SIO_status[ port ][ 1 ] < 1 );          /* wait for 1st byte (count) */
if( SIO_status[ port ][ 0 ] & 0x10 )
  return( -2 );                                 /* exit on comms error */

if( ( count = getline( port, dest ) ) < 0 )
  return( -5 );
return( count );
}

