/************************************************************************/
/*  HWInit.c      For Prototype Digital LCD controller    (79049)       */
/*                                                                      */
/*  Created: 13/12/99                                                   */
/*                                                                      */
/*  Author: Paul Carpenter                                              */
/*                                                                      */
/*  Description:                                                        */
/*                                                                      */
/* Hardware initialisation routine called by the startup code before    */
/* the sections are initialised.  Bus controller, and other essential   */
/* hardware setup code should be placed here.                           */
/*                                                                      */
/* Processor Mode:   5 (16MB address space 16 bit, with FLASH)          */
/*                                                                      */
/* Port Pin assignments Created                                         */
/*                                                                      */
/*  10/12/99 Paul Carpenter  Original Version                           */
/************************************************************************/
/*  Port  Function    Dir      Pin*/
/*  ------------------------------*/
/*  P1.0  A0          O/P      36 */
/*  P1.1  A1          O/P      37 */
/*  P1.2  A2          O/P      38 */
/*  P1.3  A3          O/P      39 */
/*  P1.4  A4          O/P      40 */
/*  P1.5  A5          O/P      41 */
/*  P1.6  A6          O/P      42 */
/*  P1.7  A7          O/P      43 */
/*  P2.0  A8          O/P      45 */
/*  P2.1  A9          O/P      46 */
/*  P2.2  A10         O/P      47 */
/*  P2.3  A11         O/P      48 */
/*  P2.4  A12         O/P      49 */
/*  P2.5  A13         O/P      50 */
/*  P2.6  A14         O/P      51 */
/*  P2.7  A15         O/P      52 */
/*  P3.0  D8          Bi-Dir   27 */
/*  P3.1  D9          Bi-Dir   28 */
/*  P3.2  D10         Bi-Dir   29 */
/*  P3.3  D11         Bi-Dir   30 */
/*  P3.4  D12         Bi-Dir   31 */
/*  P3.5  D13         Bi-Dir   32 */
/*  P3.6  D14         Bi-Dir   33 */
/*  P3.7  D15         Bi-Dir   34 */
/*  P4.0  D0          Bi-Dir   18 */
/*  P4.1  D1          Bi-Dir   19 */
/*  P4.2  D2          Bi-Dir   20 */
/*  P4.3  D3          Bi-Dir   21 */
/*  P4.4  D4          Bi-Dir   23 */
/*  P4.5  D5          Bi-Dir   24 */
/*  P4.6  D6          Bi-Dir   25 */
/*  P4.7  D7          Bi-Dir   26 */
/*  P5.0  A16         O/P      53 */
/*  P5.1  A17         O/P      54 */
/*  P5.2  A18         O/P      55 */
/*  P5.3  A19         O/P      56 */
/*  P6.0  -WAIT       I/P      58   DISABLED */
/*  P6.1  -BREQ       I/P      59   DISABLED */
/*  P6.2  FLASHRDY    I/P      60 */
/*  P6.3  -AS         O/P      69 */
/*  P6.4  -RD         O/P      70 */
/*  P6.5  -HWR        O/P      71 */
/*  P6.6  -LWR        O/P      72 */
/*  P6.7  Clock       I/P      61 */
/*  P8.0  IRQ0 spare  I/P      87 */
/*  P8.1  -IPTIMCS    O/P      88 Bank 3 (I/P Timing)   */
/*  P8.2  -CS2        O/P      89 Bank 2 (External Flash ROM */
/*  P8.3  -LD_FILT    O/P      90 Bank 1 (Filter Loader) */
/*  P8.4  Spare       I/P      91 Bank 0 (Spare)        */
/*  P9.0  TXD0        O/P      12                       */
/*  P9.1  TXD1        O/P      13                       */
/*  P9.2  RXD0        I/P      14                       */
/*  P9.3  RXD1        I/P      15                       */
/*  P9.4  -IRQ4       I/P      16 I2C Controller        */
/*  P9.5  -IRQ5       I/P      17 Input Switches        */
/*  PA.0  SCOPE0      I/P      93 In Events checking    */
/*  PA.1  SCOPE1      I/P      94 In Command Processing */
/*  PA.2  TIOCA0      I/P      95 50Hz square wave edge at each timer tick */
/*  PA.3  SCOPE2      I/P      96 In Headtracker Comms Parse */
/*  PA.4  SCOPE3      I/P      97 In Backlight PID      */
/*  PA.5  SCOPE4      I/P      98 In Error Halt loop    */
/*  PA.6  A21         O/P      99           */
/*  PA.7  A20         O/P      100          */
/*  PB.0  -I2C_CS     O/P      2  Bank 7    */
/*  PB.1  -FILT_DRQ   I/P      3            */
/*  PB.2  -OPTIMCS    O/P      4  Bank 5    */
/*  PB.3  -LUT_DRQ    I/P      5            */
/*  PB.4  Spare       I/P      6            */
/*  PB.5  Spare       I/P      7            */
/*  PB.6  TXD2        O/P      8            */
/*  PB.7  RXD2        I/P      9            */
/************************************************************************/
/* Version History                                                      */
/************************************************************************/
/* definition of the register addresses */
#include "codes.h"
#include "h83067f.h"

void hw_initialise( void ) 
{
P1DDR     = 0xFF;       /* Enable A0-A7 */
P2DDR     = 0xFF;       /* Enable A8-A15 */
P5DDR     = 0x0F;       /* Enable A16-A19 */
BRCR      = 0xC0;       /* Disable Bus release and enable A22 - A23 as ports */
BCR       = 0xC2;       /* Disable WAIT pin, no burst ROM, all banks 2MB */
                        /* Idle cycles between banks & consecutive reads */
CSCR      = 0xA0;       /* Enable CS7 and CS5 (I2C and OP Timing) */
/* Following forces P4 and P3 to be data bus by 16 bit access */
ABWCR     = 0xFD;       /* Bank 1 is 16-bit access */
ASTCR     = 0x80;       /* Bank 7, 5, 3 THREE cycle access */
WCRH      = 0xC0;       /* Bank 7 Insert three waits states & bank 5 1 wait */
WCRL      = 0x00;       /* Bank 3 insert 1 wait state */
MSTCRH    = 0x00;       /* Disable module power downs */
MSTCRL    = 0x00;

/*  If needed DCRA and DCRB are 0 to disable DRAM  */
/*  This is default condition after reset            */
DCRA      = 0;
DCRB      = 0;
/* To ensure Port B pin modes get selected correctly */
ITU8_TCSR0 = 0;
ITU8_TCSR1 = 0;
ITU8_TCSR2 = 0;
ITU8_TCSR3 = 0;

P6DDR     = 0x80;       /* Disable Bus masters Set P6 input RD/WR/AS default */
P8DDR     = 0x0E;       /* Enable CS1 to CS3 rest to inputs */
P9DDR     = 0x00;       /* Port 9 inputs (see also SIO) */
PBDDR     = 0x00;       /* Unused pins inputs */

/* Analog to Digital Converter */
ADCR      = 0;          /* Disable A/D trigger */

/* Interupt controller */
IPRA      = 0;          /* All internal devices low priority int */
IPRB      = 0;

IER       = 0x31;       /* Disable IRQs except IRQ0, 4, 5 */

/* Initialise Timer modes */
/* General timer init */
ITU_TSTR  = 0;          /* Ensure Timers stopped */
ITU_TSNC  = 0;          /* 3 asynchronous timers */
ITU_TMDR  = 0;          /* Timers operate as counters */
ITU_TISRA = 0;          /* Clear interupt status IMA */
ITU_TISRB = 0;          /* Clear interupt status IMB */
ITU_TISRC = 0;          /* Clear interupt status OVF */

/* ITU0 16bit for periodic 100Hz counter (10ms) */
ITU_TCR0  = 0x23;       /* Set counter to use system clk/8 on rising edges */
                        /* Cleared by match on GRA */
ITU_GRA0  = 25000;      /* Count value to compare for 0.01s */
ITU_TIOR0 = 0x03;       /* Toggle output at compare A match */
ITU_TOLR  = 0x0;        /* Pin 95 TIOCA0 starts at 0 */
ITU_TISRA = 0x10;       /* Enable IMIA0 interupt */
                        /* start the counting later AFTER enabling ints */
PADR      = 0x3F;       /* Set scope triggers for end h/w init (low level) */
/* DMA controller initialise */
}
