/************************************************************************
  File: SERIAL.C                for H8/3067                  (79049)

  Date: 8/5/00 PC

  Description:  Interupt I/O routines for the three Serial ports.

***** This code assumes that hardware has been configured at start_up

Utilises SIO_mode[ 3 ] in defaults section for selection of defaults for
        Baud rate
        Head tracker port
        Command port
        Port Enable
***************************************************************************/
#include <stdarg.h>
#include <ctype.h>
#include <stdlib.h>   
#include <string.h>

#include "ascii.h"
#include "h83067f.h"
#include "codes.h"
#include "h8core_b.h"
#include "parse.h"

/* The following two arrays must be of the SAME length and order */
/* Clock source assuming 20MHz clock */
BYTE baud_rates[ MAX_RATES ][ 2 ] __attribute__ ((section (".strings"))) =
            {
            { 1, 64 },     /* 2400 */
            { 0, 129 },    /* 4800 */
            { 0, 64 },     /* 9600 (use 64-20Mhz, 51-16MHz, 58-18MHz clock) */
            { 0, 32 },     /* 19200 */
            { 0, 19 },     /* 31250 */
            { 0, 15 }      /* 38400 */
            };
char   *rate_str[ MAX_RATES ] __attribute__ ((section (".strings"))) =
            {
            "2400",
            "4800",
            "9600",
            "19200",
            "31250",
            "38400"
            };
char   erase[ 4 ] __attribute__ ((section (".strings"))) =
            { BS, SPACE, BS, '\0' };                /* Backspace seq */
char   ser_ptr_str[] __attribute__ ((section (".strings"))) =
            "\n\rPtrs RX R-%lX W-%lX\n\r     TX R-%lX W-%lX";
char   ser_stat_str[] __attribute__ ((section (".strings"))) =
            "\n\rStatus %2X\n\rCount R-%d\n\rCount T-%d";

/* The following four bytes ONLY used in interupt routines  */
/* do not use for anything else                             */
BYTE    bin_count;              /* Binary byte count data count expected    */
BYTE    bin_state;              /* Binary state 0 = Character mode          */
                                /*              1 = waiting byte count      */
                                /*              2 = receiving data          */
BYTE    packet_count;           /* current count of bytes received          */
BYTE    head_packet;            /* bits 4-0 = count of end of last packet   */

volatile BYTE SIO_status[ 3 ][ 3 ];     /* Status, rx & tx counts each SIO  */
                                        /* Status is defined as:-           */
                                        /*  bit 7 = Frame/Return received   */
                                        /*  bit 6 = headtracker detected    */
                                        /*  bit 5 = Break detected          */
                                        /*  bit 4 = TX Error                */
                                        /*  bit 3 = RX Error                */
                                        /*  bit 2 = TX Bytes pending        */
                                        /*  bit 1 = RX Bytes pending        */
                                        /*  bit 0 = Mode Char(0)/Binary(1)  */
volatile BYTE *SIO_ptr[ 3 ][ 4 ];       /* Read & write pointers each SIO   */
                                        /* Buffer RX rd,wr then TX rd,wr    */

volatile struct
        {
        WORD size;                  /* Buffer size in bytes */
        BYTE *start;                /* Base address */
        BYTE *end;                  /* End address + 1 */
        } SIO_conf[ 3 ][ 2 ];       /* Config of ports buffers RX, TX   */


/***** Prototypes defined here **************/
int comm_baud( void );
int set_baud( int );
int serial_init( int );
int serial_mode( int, int );
void tx_int0( void );
void tx_int1( void );
void tx_int2( void );
void rx_int0( void );
void rx_int1( void );
void rx_int2( void );
void rx_int_err0( void );
void rx_int_err1( void );
void rx_int_err2( void );
void putchar( int, int );
int  putstring( int, char * );
int  getchar( int );
int  getline( int, char * );
int  test_serial( void );


/****************************************************************/
/*  comm_baud - Command parsing of change baud rate for SIO1    */
/*                                                              */
/*  Input       paramc of 2                                     */
/*              paramv[ 1 ] is string of baud rate to attempt   */
/*                                                              */
/*  Output      to SIO1  2 x STATUS_OK for success              */
/*              C1  for invalid param 1                         */
/*              CF  (if seen) for hardware failure              */
/*                                                              */
/*  Checks parameter against valid choices. If valid sends      */
/*  status OK as acknowledge then calls set_baud to action.     */
/*                                                              */
/*  If set_baud fails SIO1 is re-enabled and new default rate   */
/*  is only saved in RAM defaults, so a reset will recover      */
/*  situation.                                                  */
/*                                                              */
/*  Otherwise just hope that the error message can be seen!!    */
/****************************************************************/
int comm_baud( void )
{
for( loop = 0; loop < MAX_RATES; loop++ )
    if( strcmp( rate_str[ loop ], paramv[ 1 ] ) == 0 )
        break;
if( loop == MAX_RATES )
    error = BAD_PARAM1;
else
    {
    status = STAT_OK;
    send_status( );
    set_baud( loop );                /* go set the baud rate */
    }
if( error > 0 )
    return( error == HARDWARE_FAIL ? -2 : -1 );
return( 0 );
}


/****************************************************************/
/*  set_baud    Set baud rate for SIO1                          */
/*                                                              */
/*  Input       int offset into baud rate tables                */
/*                                                              */
/*  Output      error (-2 ) or  success (0)                     */
/*                                                              */
/*  Checks no characters to send, waits until they have         */
/*  completed, before next stage.                               */
/*                                                              */
/*  Disables SIO1, the tries to set the new baud rate for SIO1, */
/*  if fails sets error and exits.                              */
/*                                                              */
/*  If fails or not SIO1 is re-enabled and new deafult rate is  */
/*  saved in RAM defaults, so a reset will recover situation.   */
/*                                                              */
/*  Otherwise just hope that error message can be seen instead! */
/*                                                              */
/*  Function is called form initialisation and command to       */
/*  change baud rate if valid choice given                      */
/*                                                              */
/*  NOTE no timeout to stop hardware fault causing wait for     */
/*          transmission to end to loop forever                 */
/****************************************************************/
int set_baud( int off )
{
while( ( SCI1_SSR & 0x04 ) == 0 );              /* wait all chars sent  */
SCI1_SCR &= 0xCF;                               /* disable rx and tx    */
SCI1_SMR = ( SCI1_SMR & 0xFC ) | baud_rates[ off ][ 0 ]; /* set clock source */
SCI1_BRR = baud_rates[ off ][ 1 ];              /* set rate multiplier  */
if( ( SCI1_SMR & 0x03 ) != baud_rates[ off ][ 0 ]   /* actually did set OK  */
        || SCI1_BRR != baud_rates[ off ][ 1 ] )
    error = HARDWARE_FAIL;                      /* NO set error flag    */
SCI1_SCR |= 0x30;                               /* reenable RX & TX     */
SIO_status[ 1 ][ 0 ] =
        ( ( SIO_status[ 1 ][ 0 ] & 0xF8 ) | off );
return( error == HARDWARE_FAIL ? -2 : 0 );
}


/****************************************************************/
/* serial_init  Initialise serial ports                         */
/*                                                              */
/*  Input   int port 0 port 0 initialise                        */
/*                   1 port 1 initialise                        */
/*                   2 port 2 initialise                        */
/*                                                              */
/*  Output  Returns HARDWARE_FAIL for unable to initialise H/W  */
/*              or -1 invalid port                              */
/*                  0 for success                               */
/*                                                              */
/* Checks port is enabled in software, then configures device.  */
/* Problems with settings not matching expected cause an error  */
/* code to be returned.                                         */
/*                                                              */
/* Assumes called at startup, so characters will be lost if in  */
/* transmission or reception.                                   */
/****************************************************************/
int serial_init( int port )
{
BYTE            start, end;
volatile BYTE   *ptr;
BYTE            delay;
WORD            offset;
    
if( port < -1 || port > 2 )
  return( -1 );
if( port == -1 )
  {
  start = 0;
  end = 2;
  }
else
  start = end = port;
for( ; start <= end; start++ )
  {
  ptr = &SCI0_SMR + ( start * 8 );      /* point to base of correct SCI */
  while( !( *( ptr + 4 ) & 0x04 ) );    /* Wait for any characters to finish */
  *( ptr + 2 ) = 0;                     /* disable ints and SCIn */
  *( ptr ) = baud_rates[ 2 ][ 0 ];        /* Clock mult 9600 */
  *( ptr + 1 ) = baud_rates[ 2 ][ 1 ];    /* BRR reg    */

  /* Wait at least one bit time */
  delay = *( ptr + 5 );                 /* dummy read of two chars to clear */
  delay = *( ptr + 5 );                 /* any receive status */
  SIO_status[ start ][ 0 ] = 0;         /* Status to ASCII idle */
  SIO_status[ start ][ 1 ] = 0;         /* RX and TX counts to 0 */
  SIO_status[ start ][ 2 ] = 0;

  offset = start * ( RX_SIZE + TX_SIZE );

  /* Configure buffer parameters */
  SIO_conf[ start ][ 0 ].start = rx_buf0 + offset;
  SIO_conf[ start ][ 0 ].end   = SIO_conf[ start ][ 0 ].start + RX_SIZE;
  SIO_conf[ start ][ 0 ].size  = RX_SIZE;
  SIO_conf[ start ][ 1 ].start = tx_buf0 + offset;
  SIO_conf[ start ][ 1 ].end   = SIO_conf[ start ][ 1 ].start + TX_SIZE;
  SIO_conf[ start ][ 1 ].size  = TX_SIZE;

  /* Initialise RX and TX buffer RD/WR pointers to start of buffer */
  SIO_ptr[ start ][ 0 ] = SIO_conf[ start ][ 0 ].start;
  SIO_ptr[ start ][ 1 ] = SIO_conf[ start ][ 0 ].start;
  SIO_ptr[ start ][ 2 ] = SIO_conf[ start ][ 1 ].start;
  SIO_ptr[ start ][ 3 ] = SIO_conf[ start ][ 1 ].start;
  *( ptr + 2 ) |= 0x70;                 /* Set Rx and Tx enable, RX ints */
  }
}


/************************************************************/
/* Serial_mode set RX serial mode to ASCII or Binary        */
/*                                                          */
/*  Inputs          int port - 1 or 2 for SCI1 or SCI2      */
/*                  int mode - 0 - ASCII                    */
/*                             1 - Binary                   */
/*                                                          */
/*  Sets software modes for ASCII/Binary modes and resets   */
/*  buffer status, then repoints buffers to the appropriate */
/*  binary or ASCII buffer.                                 */
/*                                                          */
/*  Note Binary buffer is 128 bytes long for RX only        */
/*  Only port 1 or 2 can be setup for binary mode at a time */
/*  Trying to set one port for binary whilst the other is   */
/*  already in binary mode will return error of -1          */
/************************************************************/
int serial_mode( int port, int mode )
{
if( mode > 1 || mode < 0 )
  return( -1 );

switch( port )
  {
  case 1:   if( SIO_status[ 2 ][ 0 ] & 1 )
              return( -1 );
            break;
  case 2:   if( SIO_status[ 1 ][ 0 ] & 1 )
              return( -1 );
            break;
  default:  return( -1 );               /* Invalid Port number */
  }

while( SIO_status[ port ][ 2 ] != 0 );  /* wait for TX buffer to empty */
disable_int( );                         /* disable ints to be safe */
SIO_status[ port ][ 0 ] = 0;            /* Clear flags set char mode */
SIO_status[ port ][ 1 ] = SIO_status[ port ][ 2 ] = 0; /* clear counts */

/* Configure buffer parameters */
if( mode )
  {
  SIO_status[ port ][ 0 ] |= 1;         /* Set binary mode */
  SIO_conf[ port ][ 0 ].start = temp_array;
  SIO_conf[ port ][ 0 ].size  = sizeof( temp_array );
  bin_state = 1;
  }
else                                    /* character mode */
  {
  SIO_conf[ port ][ 0 ].start = rx_buf0 + ( port * ( RX_SIZE + TX_SIZE ) );
  SIO_conf[ port ][ 0 ].size  = RX_SIZE;
  bin_state = 0;
  }
SIO_conf[ port ][ 0 ].end = SIO_conf[ port ][ 0 ].start
                                    + SIO_conf[ port ][ 0 ].size;
/* Reset RX buffer read and write pointers to new buffer */
SIO_ptr[ port ][ 0 ] = SIO_conf[ port ][ 0 ].start;
SIO_ptr[ port ][ 1 ] = SIO_conf[ port ][ 0 ].start;
enable_int( );                          /* Re-enable interupts */
return( 0 );
}


/****************************************************************/
/* TX Interupt routines.     (Three one for each SCI)           */
/*                                                              */
/*  On each interupt checks that there is a character to send   */
/*  from tx_bufx, sends it decrements counter, checks for       */
/*  pointer wrap around.                                        */
/*                                                              */
/*  If there is still characters to send re-enables TX interupt */
/*  After each character is loaded TDRE is cleared to enable    */
/*  actual character transmission.                              */
/****************************************************************/
#pragma interrupt
void tx_int0( void )
{
if( SIO_status[ 0 ][ 2 ] > 0 )
  {
  SCI0_SCR = 0xF0;                      /* int enable */

  /* check for buffer wrap */
  if( SIO_ptr[ 0 ][ 2 ] >= ( tx_buf0 + TX_SIZE ) )
    SIO_ptr[ 0 ][ 2 ] = tx_buf0;
  SCI0_TDR = *SIO_ptr[ 0 ][ 2 ]++;      /* Send next character */
  SIO_status[ 0 ][ 2 ]--;               /* Dec count */
  SCI0_SSR &= 0x78;                     /* UART status bits to send char */
  }
if( SIO_status[ 0 ][ 2 ] <= 0 )         /* Finished ?? */
  {
  SIO_status[ 0 ][ 2 ] = 0;             /* to be safe */
  SIO_status[ 0 ][ 0 ] &= 0xEB;         /* clear status bit pending & error */
  SCI0_SSR &= 0xFF;                     /* to keep SCI happy */
  SCI0_SCR &= 0x70;                     /* CLEAR int enable */
  }
}


#pragma interrupt
void tx_int1( void )
{
if( SIO_status[ 1 ][ 2 ] > 0 )
  {
  SCI1_SCR = 0xF0;                      /* int enable */
  /* check for buffer wrap */
  if( SIO_ptr[ 1 ][ 2 ] == SIO_conf[ 1 ][ 1 ].end )
    SIO_ptr[ 1 ][ 2 ] = SIO_conf[ 1 ][ 1 ].start;
  SCI1_TDR = *SIO_ptr[ 1 ][ 2 ]++;      /* Send next character */
  SIO_status[ 1 ][ 2 ]--;               /* Dec count */
  SCI1_SSR &= 0x7F;                     /* UART status bits to send char */
  }
if( SIO_status[ 1 ][ 2 ] <= 0 )         /* Finished ?? */
  {
  SIO_status[ 1 ][ 2 ] = 0;             /* to be safe */
  SIO_status[ 1 ][ 0 ] &= ~0x14;        /* clear status bit pending & error */
  SIO_ptr[ 1 ][ 2 ] = SIO_ptr[ 1 ][ 3 ];
  SCI1_SSR &= 0xFF;                     /* to keep SCI happy */
  SCI1_SCR &= 0x70;                     /* CLEAR int enable */
  }
}


#pragma interrupt
void tx_int2( void )
{
if( SIO_status[ 2 ][ 2 ] > 0 )
  {
  SCI2_SCR = 0xF0;                      /* int enable */
  /* check for buffer wrap */
  if( SIO_ptr[ 2 ][ 2 ] >= SIO_conf[ 2 ][ 1 ].end )
    SIO_ptr[ 2 ][ 2 ] = SIO_conf[ 2 ][ 1 ].start;
  SCI2_TDR = *SIO_ptr[ 2 ][ 2 ]++;      /* Send next character */
  SIO_status[ 2 ][ 2 ]--;               /* Dec count */
  SCI2_SSR &= 0x78;                     /* UART status bits to send char */
  }
if( SIO_status[ 2 ][ 2 ] <= 0 )         /* Finished ?? */
  {
  SIO_status[ 2 ][ 2 ] = 0;             /* to be safe */
  SIO_status[ 2 ][ 0 ] &= 0xEB;         /* clear status bit pending & error */
  SCI2_SSR &= 0xFF;                     /* to keep SCI happy */
  SCI2_SCR &= 0x70;                     /* CLEAR int enable */
  }
}


/************************************************************/
/* RX Interupt routines.                                    */
/*                                                          */
/*  On each interupt checks mode for binary character,      */
/*  (headtracker always binary).                            */
/*                                                          */
/*  If a character to receive rx_bufx, checks buffer not    */
/*  full (sets RX error & discards character).              */
/*                                                          */
/*  For character mode checks character is not <CR> or <BS> */
/*  then puts character in buffer, decrements counter,      */
/*  checks for pointer wrap around, then copies it to       */
/*  transmitter buffer for echoing.                         */
/*                                                          */
/*  In character mode receipt of <CR>, discards character,  */
/*  then sets Frame received bit. Whilst <BS> causes the    */
/*  character sequence <BS><SP><BS> to be sent to tx_buf.   */
/*                                                          */
/*  In binary mode (non-Headtracker) the stages are:-       */
/*      Waiting for '*'             (ignore other chars)    */
/*      Receive byte count                                  */
/*      Receive count bytes -2      (add to checksum)       */
/*      Receive Checksum            (error if different)    */
/*      Receive end of record '#'   (error if not found)    */
/*                                                          */
/*  All errors force state to Waiting for '*', set error bit*/
/*  and clear the buffer of that packet.                    */
/*                                                          */
/*  Only one of ports 1 and 2 can use binary mode at a time */
/*  as they both use temp_array                             */
/************************************************************/
#pragma interrupt
void rx_int0( void )                    /* Headtracker Comms        */
{
SIO_status[ 0 ][ 0 ] &= 0xF6;           /* Clear old error flags    */
if( SIO_status[ 0 ][ 1 ] >= ( SIO_conf[ 0 ][ 0 ].size - 1 ) )
  SIO_status[ 0 ][ 0 ] |= 0x08;         /* set error BUFFER OVERRUN */
else
  {
  /* Check reached end of buffer */
  if( SIO_ptr[ 0 ][ 1 ] == SIO_conf[ 0 ][ 0 ].end )
    SIO_ptr[ 0 ][ 1 ] = SIO_conf[ 0 ][ 0 ].start;
  *SIO_ptr[ 0 ][ 1 ] = SCI0_RDR;        /* read the byte            */
  SCI0_SSR &= 0xB8;                     /* clear receive done       */
  if( head_packet > 1 )                 /* part way through packet  */
    if( *SIO_ptr[ 0 ][ 1 ] == 0xFE )    /* if character == 0xFE     */
      {
      /* reset to start of current packet */
      SIO_status[ 0 ][ 1 ] -= ( head_packet );
      SIO_ptr[ 0 ][ 1 ] -= head_packet;
      if( SIO_ptr[ 0 ][ 1 ] < SIO_conf[ 0 ][ 0 ].start )
        SIO_ptr[ 0 ][ 1 ] += ( SIO_conf[ 0 ][ 0 ].size );
      head_packet = 0;
      }
  /* Update received counters etc. for valid bytes after start of   */
  /* oxFE. Ignore bytes until 0xFE if waiting for start of packet   */
  if( !( ( head_packet == 0 ) && ( *SIO_ptr[ 0 ][ 1 ] != 0xFE ) ) )
    {
    head_packet++;                      /* one more byte in packet  */
    SIO_ptr[ 0 ][ 1 ]++;                    /* inc wr pointer       */
    SIO_status[ 0 ][ 1 ]++;                 /* inc byte counter     */
    if( head_packet == 13 )                 /* Received a frame ??  */
      {
      SIO_status[ 0 ][ 0 ] |= 0x80;     /* Set frame received flag  */
      head_packet = 0;                  /* set waiting next packet  */
      }
    }
  }
}


#pragma interrupt
void rx_int1( void )                        /* Debug/Flash Comms */
{
SIO_status[ 1 ][ 0 ] &= 0xF7;               /* Clear old error flags */
if( SIO_status[ 1 ][ 0 ] & 1 )
  if( SIO_status[ 1 ][ 1 ] == SIO_conf[ 1 ][ 0 ].size - 32 )
    putchar( 1, DC3 );                      /* XOFF Binary buffer full  */
else
  if( SIO_status[ 1 ][ 1 ] >= SIO_conf[ 1 ][ 0 ].size - 1 )
    if( SCI1_RDR != CR && SCI1_RDR != BS )
      {
      SIO_status[ 1 ][ 0 ] |= 0x08;         /* set error BUFFER OVERRUN     */
      putchar( 1, BEL );                    /* Send BELL code               */
      SCI1_SSR &= 0xB8;                     /* clear receive done           */
      }
if( SIO_ptr[ 1 ][ 1 ] == SIO_conf[ 1 ][ 0 ].end )
  SIO_ptr[ 1 ][ 1 ] = SIO_conf[ 1 ][ 0 ].start; /* Check for end of buffer  */
*SIO_ptr[ 1 ][ 1 ] = SCI1_RDR;              /* read the byte                */
SCI1_SSR &= 0xB8;                           /* clear receive done           */
SIO_status[ 1 ][ 1 ]++;                     /* one more received            */

if( !( SIO_status[ 1 ][ 0 ] & 1 ) )         /* Character mode               */
  {
  if( *SIO_ptr[ 1 ][ 1 ] == BS              /* Backspace character ??       */
       || ( *SIO_ptr[ 1 ][ 1 ] == CR
             && SIO_status[ 1 ][ 1 ] == 1 ) )
    {                                       /* or CR as first character     */
    if( SIO_status[ 1 ][ 1 ] > 1 )          /* Only if chars to erase       */
      {
      putstring( 1, erase );                /* Send BS SP BS sequence       */
      SIO_ptr[ 1 ][ 1 ]--;                  /* remove BS & earlier char     */
      SIO_status[ 1 ][ 1 ] -= 2;            /* two less in buffer           */
      }
    else                                    /* BS or CR first character     */
      SIO_status[ 1 ][ 1 ]--;               /* ONE less in buffer           */
    if( SIO_ptr[ 1 ][ 1 ] < SIO_conf[ 1 ][ 0 ].start  ) /* before start?    */
      SIO_ptr[ 1 ][ 1 ] = SIO_conf[ 1 ][ 0 ].start; /* reset to buffer end  */
                                            /* I.E. point before BS & char  */
    if( SIO_status[ 1 ][ 1 ] <= 0 )         /* Is buffer now empty ??       */
      {                                     /* Yes clear flags etc..        */
      SIO_status[ 1 ][ 0 ] &= 0xFD;         /* Clear RX bytes pending status*/
      SIO_status[ 1 ][ 1 ] = 0;             /* safety reset counters & ptrs */
      SIO_ptr[ 1 ][ 0 ]
                        = SIO_ptr[ 1 ][ 1 ]
                        = SIO_conf[ 1 ][ 0 ].start ;
      }
    }
  else
    {
    if( *SIO_ptr[ 1 ][ 1 ] != CR )
      {
      *SIO_ptr[ 1 ][ 1 ] = toupper( *SIO_ptr[ 1 ][ 1 ] ); /* force upper */
      putchar( 1, *SIO_ptr[ 1 ][ 1 ] );     /* Echo character NOT BS or CR  */
      }
    else
      if( SIO_status[ 1 ][ 1 ] > 1 )
        SIO_status[ 1 ][ 0 ] |= 0x80;       /* Set frame rxed flag ( >1 )   */
    SIO_ptr[ 1 ][ 1 ]++;                    /* Increment RX Write pointer   */
    }
  }
else
  {
  if( bin_state == 1 )                      /* Waiting for byte count       */
    {
    bin_state = 2;                          /* bump state to data rx        */
    packet_count = 0;                       /* So far zero data in packet   */
    bin_count = *SIO_ptr[ 1 ][ 1 ];         /* Save value as expected count */
    }
  else
    if( ++packet_count == bin_count )       /* received the packet size ??  */
      {
      bin_state = 1;                        /* set awaiting byte count mode */
      SIO_status[ 1 ][ 0 ] |= 0x80;         /* Set frame received flag      */
      }
  SIO_ptr[ 1 ][ 1 ]++;                      /* Increment RX Write pointer   */
  }
}


#pragma interrupt
void rx_int2( void )                        /* Expansion Comms */
{
SIO_status[ 2 ][ 0 ] &= 0xF7;               /* Clear old error flags */
if( SIO_status[ 2 ][ 0 ] & 1 )
  if( SIO_status[ 2 ][ 1 ] == SIO_conf[ 2 ][ 0 ].size - 32 )
    putchar( 2, DC3 );                      /* XOFF Binary buffer full  */
else
  if( SIO_status[ 2 ][ 1 ] >= SIO_conf[ 2 ][ 0 ].size - 1 )
    if( SCI2_RDR != CR && SCI2_RDR != BS )
      {
      SIO_status[ 2 ][ 0 ] |= 0x08;         /* set error BUFFER OVERRUN     */
      putchar( 2, BEL );                    /* Send BELL code               */
      SCI2_SSR &= 0xB8;                     /* clear receive done           */
      }
if( SIO_ptr[ 2 ][ 1 ] == SIO_conf[ 2 ][ 0 ].end  )
  SIO_ptr[ 2 ][ 1 ] = SIO_conf[ 2 ][ 0 ].start; /* Check for end of buffer  */
*SIO_ptr[ 2 ][ 1 ] = SCI2_RDR;              /* read the byte                */
SCI2_SSR &= 0xB8;                           /* clear receive done           */
SIO_status[ 2 ][ 1 ]++;                     /* one more received            */

if( !( SIO_status[ 2 ][ 0 ] & 1 ) )         /* Character mode               */
  {
  if( *SIO_ptr[ 2 ][ 1 ] == BS              /* Backspace character ??       */
       || ( *SIO_ptr[ 2 ][ 1 ] == CR
             && SIO_status[ 2 ][ 1 ] == 1 ) )
    {                                       /* or CR as first character     */
    if( SIO_status[ 2 ][ 1 ] > 1 )          /* Only if chars to erase       */
      {
      putstring( 2, erase );                /* Send BS SP BS sequence       */
      SIO_ptr[ 2 ][ 1 ]--;                  /* remove BS & earlier char     */
      SIO_status[ 2 ][ 1 ] -= 2;            /* two less in buffer           */
      }
    else                                    /* BS or CR first character     */
      SIO_status[ 2 ][ 1 ]--;               /* ONE less in buffer           */
    if( SIO_ptr[ 2 ][ 1 ] < SIO_conf[ 2 ][ 0 ].start  ) /* before start?    */
      SIO_ptr[ 2 ][ 1 ] = SIO_conf[ 2 ][ 0 ].start; /* reset to buffer end  */
                                            /* I.E. point before BS & char  */
    if( SIO_status[ 2 ][ 1 ] <= 0 )         /* Is buffer now empty ??       */
      {                                     /* Yes clear flags etc..        */
      SIO_status[ 2 ][ 0 ] &= 0xFD;         /* Clear RX bytes pending status*/
      SIO_status[ 2 ][ 1 ] = 0;             /* safety reset counters & ptrs */
      SIO_ptr[ 2 ][ 0 ]
                        = SIO_ptr[ 2 ][ 1 ]
                        = SIO_conf[ 2 ][ 0 ].start ;
      }
    }
  else
    {
    if( *SIO_ptr[ 2 ][ 1 ] != CR )
      {
      *SIO_ptr[ 2 ][ 1 ] = toupper( *SIO_ptr[ 2 ][ 1 ] ); /* force upper */
      putchar( 2, *SIO_ptr[ 2 ][ 1 ] );     /* Echo character NOT BS or CR  */
      }
    else
      if( SIO_status[ 2 ][ 1 ] > 1 )
        SIO_status[ 2 ][ 0 ] |= 0x80;       /* Set frame rxed flag ( >1 )   */
    SIO_ptr[ 2 ][ 1 ]++;                    /* Increment RX Write pointer   */
    }
  }
else
  {
  if( bin_state == 1 )                      /* Waiting for byte count       */
    {
    bin_state++;                            /* bump state to data rx        */
    packet_count = 0;                       /* So far zero data in packet   */
    bin_count = *SIO_ptr[ 2 ][ 1 ];         /* Save value as expected count */
    }
  else
    if( ++packet_count == bin_count )       /* received the packet size ??  */
      {
      bin_state = 1;
      SIO_status[ 2 ][ 0 ] |= 0x80;         /* Set frame received flag      */
      }
  SIO_ptr[ 2 ][ 1 ]++;                      /* Increment RX Write pointer   */
  }
}


#pragma interrupt
void rx_int_err0( void )                /* Headtracker Comms */
{
SIO_status[ 0 ][ 0 ] |= 0x08;           /* Set error flag in s/w status */
if( SIO_ptr[ 0 ][ 1 ] == SIO_conf[ 0 ][ 0 ].end )
  SIO_ptr[ 0 ][ 1 ] == SIO_conf[ 0 ][ 0 ].start;
if( SCI0_SSR & 0x28 )                   /* Overrun and/or parity error set */
  *SIO_ptr[ 0 ][ 0 ] = SCI0_RDR;        /* discard character */
if( SCI0_SSR & 0x10 )                   /* Frame error */
  {
  SIO_status[ 0 ][ 0 ] |= 0x20;         /* Set as break detected and */
  SCI0_SCR &= 0xBF;                     /* Disable receiver */
  }
SCI0_SSR &= 0x87;                       /* clear error flags */
}


#pragma interrupt
void rx_int_err1( void )                /* Debug Comms */
{                                   
SIO_status[ 1 ][ 0 ] |= 0x08;           /* Set error flag in s/w status */
if( SIO_ptr[ 1 ][ 1 ] == SIO_conf[ 1 ][ 0 ].end )
  SIO_ptr[ 1 ][ 1 ] == SIO_conf[ 1 ][ 0 ].start;
if( SCI1_SSR & 0x28 )                   /* Overrun and/or parity error set */
  *SIO_ptr[ 1 ][ 1 ] = SCI1_RDR;        /* discard character */
if( SCI1_SSR & 0x10 )                   /* Frame error */
  {
  SIO_status[ 1 ][ 0 ] |= 0x20;         /* Set as break detected and */
  SCI1_SCR &= 0xBF;                     /* Disable receiver */
  }
SCI1_SSR &= 0x87;                       /* clear error flags */
}


#pragma interrupt
void rx_int_err2( void )                /* Expansion Comms */
{
SIO_status[ 2 ][ 0 ] |= 0x08;           /* Set error flag in s/w status */
if( SIO_ptr[ 2 ][ 1 ] == SIO_conf[ 2 ][ 0 ].end )
  SIO_ptr[ 2 ][ 1 ] == SIO_conf[ 2 ][ 0 ].start;
if( SCI2_SSR & 0x28 )                   /* Overrun and/or parity error set */
  *SIO_ptr[ 2 ][ 1 ] = SCI0_RDR;        /* discard character */
if( SCI2_SSR & 0x10 )                   /* Frame error */
  {
  SIO_status[ 2 ][ 0 ] |= 0x20;         /* Set as break detected and */
  SCI2_SCR &= 0xBF;                     /* Disable receiver */
  }
SCI2_SSR &= 0x87;                       /* clear error flags */
}


/************************************************************/
/* Put character into serial port specified's TX buffer     */
/*  DISCARD character if buffer full                        */
/************************************************************/
void putchar( int port, int ch )
{
volatile BYTE   *ptr;

/* wait for space to send */
while( SIO_status[ port ][ 2 ] >= SIO_conf[ port ][ 1 ].size );

if( !( SIO_status[ port ][ 0 ] & 0x04 ) )  /* no bytes waiting */
  {                                     /* preload first character and send */
  SIO_status[ port ][ 0 ] |= 0x04;      /* set bytes pending (sending) */
  SIO_status[ port ][ 2 ] = 0;          /* ensure count starts at zero */
                                        /* as nothing in buffer */
  ptr = &SCI0_SMR + ( port * 8 );       /* point to base of correct SCI */
  *( ptr + 3 ) = ch;                    /* TDR preload first character */
  *( ptr + 4 ) &= 0x74;                 /* SSR clear TDRE */
  *( ptr + 2 ) = 0xF0;                  /* SCR interupt enable */
  }
else
  {
  if( SIO_ptr[ port ][ 3  ] == SIO_conf[ port ][ 1 ].end )
    SIO_ptr[ port ][ 3 ] = SIO_conf[ port ][ 1 ].start;
  *( SIO_ptr[ port ][ 3 ]++ ) = ch;     /* Copy character */
  SIO_status[ port ][ 2 ]++;            /* Inc count */
  }
}


/************************************************************/
/* Put string into serial port specified's TX buffer        */
/* DISCARD characters if buffer full                        */
/************************************************************/
int putstring( int port, char *buf )
{
int ret_val;
while( *buf != '\0' )
     {
     putchar( port, (int)*buf++ );
     ret_val++;
     }
return( ret_val );
}


/************************************************************/
/* Get a character from serial port specified's RX buffer   */
/*  Return error of -1 if none available                    */
/************************************************************/
int getchar( int port )
{
if( SIO_status[ port ][ 1 ] > 0 )               /* something to read ?? */
  {
  if( SIO_ptr[ port ][ 0 ] == SIO_conf[ port ][ 0 ].end )
    SIO_ptr[ port ][ 0 ] = SIO_conf[ port ][ 0 ].start;
  disable_int( );
  SIO_status[ port ][ 1 ]--;                    /* Decrement count */
  enable_int( );
  if( SIO_status[ port ][ 0 ] & 1 )             /* Binary XON handler */
    if( ( SIO_status[ port ][ 1 ] == ( SIO_conf[ port ][ 0 ].size - 17 ) ) )
      putchar( port, DC1 );
  return( *SIO_ptr[ port ][ 0 ]++ );            /* return next character */
  }
return( -1 );
}


/************************************************************/
/* get a line upto CR from the buffer, return count of chars*/
/* or -1 for error.                                         */
/************************************************************/
int getline( int port, char *buf )
{
int cnt, cnt1, val;

if( !( SIO_status[ port ][ 0 ] & 0x01 ) )
  {                                         
  cnt = -1;                                     /* Character mode           */
  while( ( *( buf ) = (char)( getchar( port ) ) ) != (char) -1 )
    {
    cnt++;
    if( *( buf++ ) == CR )
      break;
    }
  if( *( buf - 1 ) == (char) -1 )
    {
    cnt = -1;                                   /* return error flag        */
    buf--;                                      /* make sure overwrite -1   */
    }
  *( buf ) = '\0';
  }
else
  {                                             /* Binary mode              */
  if( ( cnt = getchar( port ) ) < 1 )           /* error or count of zero   */
    return( -1 );
  while( SIO_status[ port ][ 1 ] < cnt );       /* make sure full packet    */
  for( cnt1 = 0; cnt1 < cnt; cnt1++ )
     {
     if( ( val = getchar( port ) ) == -1 )
       return( -1 );
     *buf++ = (BYTE)val;
     }
  }
if( ( SIO_status[ port ][ 1 ] == 0 )            /* nothing in buffer        */
      || ( SIO_status[ port ][ 1 ] > 0          /* or incomplete next packet */
           && *SIO_ptr[ port ][ 0 ] < ( SIO_status[ port ][ 1 ] - 1 ) ) )
  SIO_status[ port ][ 0 ] &= ~0x80;             /* Clear packet waiting flag*/
return( cnt );
}


/************************************************/
/* test function                                */
/* print out of SIO1 ptrs and status for debug  */
/************************************************/
int test_serial( void )
{
for( loop = 0; loop < 3; loop++ )
   {
   temp_numb[ loop ].ch = SIO_status[ 0 ][ loop ];
   temp_addr[ loop ] = (unsigned long)SIO_ptr[ 0 ][ loop ];
   }
temp_addr[ 3 ] = (unsigned long)SIO_ptr[ 0 ][ loop ];
mon_printf( ser_ptr_str, temp_addr[ 0 ], temp_addr[ 1 ],
                                        temp_addr[ 2 ], temp_addr[ 3 ] );
mon_printf( ser_stat_str, temp_numb[ 0 ].ch, temp_numb[ 1 ].ch,
                                        temp_numb[ 2 ].ch );
return( 0 );
}

