#include    <string.h>
#include    "codes.h"
#include    "h8core_b.h"
#include    "parse.h"
#include    "console.h"
#include    "serial.h"
#include    "flash.h"

/***************** Prototypes ***************************/
int check_add( unsigned int, int );
int ret_normal( void );
int write_mem( void );
int read_mem( void );
int comm_setup( void );
int get_mem_data( BYTE *, int, int );

/********************************************************/
/* Ret_normal   return to normal mode from Setup mode   */
/*                                                      */
/*  Input parameters  NONE                              */
/*                                                      */
/*  Error conditions  NONE                              */
/*                                                      */
/*  Return parameter  integer                           */
/*                     0 = Success                      */
/*                                                      */
/*  Function acknowledges command with setup mode       */
/*  status of OK (SETUP_OK) then switches modes         */
/*  returning with no error.                            */
/*  This will re-enable backlight processing            */
/*                                                      */
/*  Returning with no error and mode change will cause  */
/*  the calling section to send status AGAIN but this   */
/*  time in NORMAL mode.                                */
/********************************************************/
int ret_normal( void )
{
status = SETUP_END;
send_status( );
mode = NORMAL;
error = 0;
return( 0 );
}


/********************************************************/
/* Comm_setup   Enter Setup mode                        */
/*                                                      */
/*  Input parameters  NONE                              */
/*                                                      */
/*  Error conditions  NONE                              */
/*                                                      */
/*  Return parameter  integer                           */
/*                     0 = Success                      */
/*                                                      */
/*  Function acknowledges command with normal mode      */
/*  status of OK (STAT_OK) then ensures backlights set  */
/*  to min as they have no processing time in these     */
/*  functions. Switches modes returning with no error.  */
/*                                                      */
/*  Returning with no error and mode change will cause  */
/*  the calling section to send status AGAIN but this   */
/*  time in SETUP mode.                                 */
/********************************************************/
int comm_setup( void )
{
status = STAT_OK;
send_status( );
error = 0;
mode = SETUP;
return( 0 );
}


/********************************************************/
/* write_mem    Write host data to memory block         */
/*                                                      */
/*  Input parameters unsigned int type - Range 0 to 2   */
/*                                                      */
/*  Error conditions  Not a number or out of range      */
/*                                                      */
/*  Return parameter  integer                           */
/*                    -1 = Invalid/not allowed address  */
/*                    -2 = Invalid type parameter       */
/*                     0 = Success                      */
/********************************************************/
int write_mem( void )
{
BYTE    *ptr;
unsigned long   count;
    
count = 0;
if( paramc == 2 )                               /* Send only one location */
  {
  if( ( error = -check_add( WRITE, 1 ) ) == 0 )  /* convert address */
    get_mem_data( (BYTE *)temp_addr[ 0 ], 1, 1 );
  }
else
  {
  /* Receive binary encoded block of more than 1 value */
  if( ( error = -check_add( WRITE, 1 ) ) == 0 )     /* start address */
    if( ( error = -check_add( WRITE, 2 ) ) == 0 )   /* end address */
      if( temp_addr[ 1 ] > temp_addr[ 0 ] )     /* valid address range */
        count = get_mem_data( (BYTE *)temp_addr[ 0 ],
                                temp_addr[ 1 ] - temp_addr[ 0 ] + 1, 1 );
      else
        error = 1;
  }
if( error != 0 )
  return( -2 );
return( 0 );                                 /* Did an error occur?? */
}


/********************************************************/
/* Read_mem Read the block of memory and send to host   */
/*                                                      */
/*  Input parameters paramv[ 1 ] is source address      */
/*                      0 = Read address          (READ)*/
/*                      1 = Write address        (WRITE)*/
/*                      2 = Flash write address  (FLASH)*/
/*                   char *param null termiated string  */
/*                      of ASCII HEX address to convert */
/*                                                      */
/*  Error conditions  Not a number or out of range      */
/*                                                      */
/*  Return parameter  integer                           */
/*                    -1 = Invalid/not allowed address  */
/*                    -2 = Invalid type parameter       */
/*                     0 = Success                      */
/********************************************************/
int read_mem( void )
{
if( paramc == 2 )                               /* Send only one location */
  {
  if( ( error = -check_add( READ, 1 ) ) == 0 )   /* convert read address */
    {
    /* send values */
    mon_printf( " = %2X ( %d )", *(BYTE *)temp_addr[ 0 ],
                                    *(BYTE *)temp_addr[ 0 ] );
    }
  }
else
  {
  /* Send binary encoded block of more than 1 value */
  if( ( error = -check_add( READ, 1 ) ) == 0 )   /* start address */
    if( ( error = -check_add( READ, 2 ) ) == 0 ) /* end address */
      if( temp_addr[ 1 ] > temp_addr[ 0 ] )     /* valid address range */
        error = send_hex_block( temp_addr[ 0 ],
                    (unsigned long)(temp_addr[ 1 ] - temp_addr[ 0 ] ) + 1, 1 );
      else
        error = 1;
  }
if( error != 0 )
  return( -1 );
return( 0 );
}


/********************************************************/
/* Check_add  Parameter address check                   */
/*                                                      */
/*  Input parameters unsigned int type - Range 0 to 2   */
/*                      0 = Read address          (READ)*/
/*                      1 = Write address        (WRITE)*/
/*                      2 = Flash write address  (FLASH)*/
/*                   char *param null termiated string  */
/*                      of ASCII HEX address to convert */
/*                   int parameter number               */
/*  Error conditions  Not a number or out of range      */
/*                                                      */
/*  Return parameter  integer                           */
/*                    -1 = Invalid/not allowed address  */
/*                    -2 = Invalid type parameter       */
/*                    -3 = NULL parameter address       */
/*                    -4 = Not allowed I/O address      */
/*                    -5 = Address above int RAM        */
/*                    -6 = Address between Flash blocks */
/*                    -7 = Write to non-RAM area        */
/*                    -8 = Address beyond FLASH RAM     */
/*                     0 = Success                      */
/*                                                      */
/*  For ALL commands using this routine the syntax of   */
/*  address parameters is that parameter order (n) is   */
/*                                                      */
/*  0   Command type                                    */
/*  1   Start address               (temp_addr[ 0 ])    */
/*  2   End address                 (temp_addr[ 1 ])    */
/*  3   Destination start address   (temp_addr[ 2 ])    */
/*                                                      */
/* Description                                          */
/*   Take the parameter paramv[ n ], convert to address */
/*   and store in temp_add [ n - 1 ] and checks it is   */
/*   in range for the type of operation requested.      */
/********************************************************/
int check_add( unsigned int type, int n )
{
unsigned long return_add;               /* computed value */

if( type > FLASH_T )
  return( -2 );                         /* Invalid argument return error */
if( paramv[ n ] == NULL )
  return( -3 );

/* convert hex address as 32 bit number */
if( conv_address( n ) )
  return( -2 );
return_add = temp_addr[ n - 1 ];        /* fetch return address to use */

/* First check for valid ranges used */
if( return_add > 0xFFFFFF )             /* only positive 24 bit addresses */
  return( -1);
if( return_add > (unsigned long)&RAM_TOP )
  return( -5 );                         /* No direct access to registers */
if( return_add >= ( (unsigned long)&check_sum + 4 ) )
  return( -6 );                         /* Nothing between int & ext Flash */

/* Write mode address range checks Internal RAM only */
if( type == WRITE && return_add < (unsigned long)&RAM_START )
  return( -7 );

return( 0 );                            /* If got here address is VALID */
}


int get_mem_data( BYTE *start, int size, int port )
{
BYTE *ptr;
int count;

ptr = start;
serial_mode( port, 1 );                    /* Set port binary mode */
while( ptr < ( start + size ) )
     {
     if( ( count = get_hex_packet( temp_array1, 1 ) ) > 0 )
       {
       memcpy( ptr, temp_array1, count );    /* save packet at a time */
       ptr += count;                         /* pointer inc */
       }
     else                                         /* Trap comms errors */
       if( count < 0 )
         {
         error = 1;
         break;
         }
     }
putstring( 1, "\n\rGot data packets" );
while( SIO_status[ 1 ][ 0 ] & 0x04 );
serial_mode( port, 0 );                    /* set port ASCII mode */
if( error != 0 )
  {
  switch( count )
        {
        case 0:  error = 0xE3;           /* Invalid character sequence */
                 break;
        case -1: error = 4;             /* Invalid parameter */
                 break;
        case -2: error = 0xE2;          /* Comms error */
                 break;
        }
  return( -2 );
  }
return( 0 );
}

