/*;***********************************************************************
sci.c
  H8/300H SCI-0 & SCI-1 serial I/O routines

  Copyright 1999, R O SoftWare

*************************************************************************/

#include   "h3003.h"
#include   "sci.h"


/* === local vars ===================================================== */

#if SCI0_SUPPORT
static unsigned char  sci0_rx_buffer[SCI0_RX_BUFFER_SIZE];
static unsigned short sci0_rx_insert_idx, sci0_rx_extract_idx;

static unsigned char  sci0_tx_buffer[SCI0_TX_BUFFER_SIZE];
static unsigned short sci0_tx_insert_idx, sci0_tx_extract_idx;
#endif

#if SCI1_SUPPORT
static unsigned char  sci1_rx_buffer[SCI1_RX_BUFFER_SIZE];
static unsigned short sci1_rx_insert_idx, sci1_rx_extract_idx;

static unsigned char  sci1_tx_buffer[SCI1_TX_BUFFER_SIZE];
static unsigned short sci1_tx_insert_idx, sci1_tx_extract_idx;
#endif


#if SCI0_SUPPORT

/* Initialize sci0 for async mode
 * typical call - sci0_init(BAUD(9600), SCI_8N1);
 *           or - sci0_init(B9600, SCI_8N1);
 */
void sci0_init(unsigned char baud, unsigned char config)
{
  SCR0 = 0x00;                          // clear TE & RE in SCR
  SMR0 = config;                        // set char length, parity, & # of stop bits
  BRR0 = baud;                          // set baud rate

  /* "Empty" the character buffers. */
  sci0_rx_insert_idx = sci0_rx_extract_idx = 0;
  sci0_tx_insert_idx = sci0_tx_extract_idx = 0;

  SCR0 = 0xF0;                          // TIE, RIE, TE, RE, !MPIE, !TEIE, sysclk
}


/* Send a character via a transmit channel.
 * Return 0 on success, -1 on error (queue full).
 */
int sci0_putc(char ch)
{
  unsigned short temp;

  temp = (sci0_tx_insert_idx + 1) % SCI0_TX_BUFFER_SIZE;

  if (temp == sci0_tx_extract_idx)
    return -1;                          // no room

  sci0_tx_buffer[sci0_tx_insert_idx] = (unsigned char)ch;
  sci0_tx_insert_idx = temp;

  asm(" bset #7,@0xFFFFB2:8");          // enable TRDE intrpts, (set TIE in SCR0)
  return 0;
}


/* get available space in sci transmit queue
 */
unsigned short sci0_space(void)
{
  int space;

  if ((space = (sci0_tx_extract_idx - sci0_tx_insert_idx)) <= 0)
    space += SCI0_TX_BUFFER_SIZE;

  return (unsigned short)(space - 1);
}


/* write NULL terminated 'string' to the sci output
 * queue, returning a pointer to the next character
 * to be written.  (\0 if full string is written)
 */
char *sci0_puts(char *string)
{
  register char ch;

  while ((ch = *string) && (sci0_putc(ch) == 0))
    string++;

  return string;
}


/* Write 'count' characters from 'buffer' to the sci
 * output queue.
 * Return 0 on success, -1 if insufficient room, -2 on error
 * NOTE: if insufficient room, no characters are written.
 */
int sci0_write(char *buffer, unsigned short count)
{
  if (count > sci0_space())
    return -1;

  while (count && (sci0_putc(*buffer++) == 0))
    count--;

  return (count ? -2 : 0);
}


/* Get a character via a receive channel.
 * Return character on success, -1 if no character is available.
 */
int sci0_getc(void)
{
  unsigned char ch;

  if (sci0_rx_insert_idx == sci0_rx_extract_idx) // check if character is available
    return -1;

  ch = sci0_rx_buffer[sci0_rx_extract_idx++]; // get character, bump pointer
  sci0_rx_extract_idx %= SCI0_RX_BUFFER_SIZE; // limit the pointer
  return ch;
}


/* sci0 receive isr
 * direct sci0 receive error vector to here also
 */
#pragma interrupt
void sci0_rcv_isr(void)
{
  unsigned char status;
  unsigned short temp;

  status = SSR0;                        // get status register
  SSR0 = status & 0x87;                 // clear RDRF & error flags

  if (status & 0x40)                    // check RDRF bit
    {
    temp = (sci0_rx_insert_idx + 1) % SCI0_RX_BUFFER_SIZE;
    sci0_rx_buffer[sci0_rx_insert_idx] = RDR0; // read the character

    if (temp != sci0_rx_extract_idx)
      sci0_rx_insert_idx = temp;
    }
}


/* sci0 transmit isr
 */
#pragma interrupt
void sci0_xmt_isr(void)
{
  if (sci0_tx_insert_idx != sci0_tx_extract_idx)
    {
    TDR0 = sci0_tx_buffer[sci0_tx_extract_idx++];
    sci0_tx_extract_idx %= SCI0_TX_BUFFER_SIZE;

    /* clear this interrupt */
    asm(" bld  #7,@0xFFFFB4:8");        // read SSR0
    asm(" bist #7,@0xFFFFB4:8");        // clear TDRE
    }
  else
    {
    /* disable transmit interrupts */
    asm(" bclr #7,@0xFFFFB2:8");        // clear TIE in SCR0
    }
}

#endif


#if SCI1_SUPPORT

/* Initialize sci1 for async mode
 * typical call - sci1_init(BAUD(9600), SCI_8N1);
 *           or - sci1_init(B9600, SCI_8N1);
 */
void sci1_init(unsigned char baud, unsigned char config)
{
  SCR1 = 0x00;                          // clear TE & RE in SCR
  SMR1 = config;                        // set char length, parity, & # of stop bits
  BRR1 = baud;                          // set baud rate

  /* "Empty" the character buffers. */
  sci1_rx_insert_idx = sci1_rx_extract_idx = 0;
  sci1_tx_insert_idx = sci1_tx_extract_idx = 0;

  SCR1 = 0xF0;                          // TIE, RIE, TE, RE, !MPIE, !TEIE, sysclk
}


/* Send a character via a transmit channel.
 * Return 0 on success, -1 on error (queue full).
 */
int sci1_putc(char ch)
{
  unsigned short temp;

  temp = (sci1_tx_insert_idx + 1) % SCI1_TX_BUFFER_SIZE;

  if (temp == sci1_tx_extract_idx)
    return -1;                          // no room

  sci1_tx_buffer[sci1_tx_insert_idx] = (unsigned char)ch;
  sci1_tx_insert_idx = temp;

  asm(" bset #7,@0xFFFFBA:8");          // enable TRDE intrpts, (set TIE in SCR1)
  return 0;
}


/* get available space in sci transmit queue
 */
unsigned short sci1_space(void)
{
  int space;

  if ((space = (sci1_tx_extract_idx - sci1_tx_insert_idx)) <= 0)
    space += SCI1_TX_BUFFER_SIZE;

  return (unsigned short)(space - 1);
}


/* write NULL terminated 'string' to the sci output
 * queue, returning a pointer to the next character
 * to be written.  (\0 if full string is written)
 */
char *sci1_puts(char *string)
{
  register char ch;

  while ((ch = *string) && (sci1_putc(ch) == 0))
    string++;

  return string;
}


/* Write 'count' characters from 'buffer' to the sci
 * output queue.
 * Return 0 on success, -1 if insufficient room, -2 on error
 * NOTE: if insufficient room, no characters are written.
 */
int sci1_write(char *buffer, unsigned short count)
{
  if (count > sci1_space())
    return -1;

  while (count && (sci1_putc(*buffer++) == 0))
    count--;

  return (count ? -2 : 0);
}


/* Get a character via a receive channel.
 * Return character on success, -1 if no character is available.
 */
int sci1_getc(void)
{
  unsigned char ch;

  if (sci1_rx_insert_idx == sci1_rx_extract_idx) // check if character is available
    return -1;

  ch = sci1_rx_buffer[sci1_rx_extract_idx++]; // get character, bump pointer
  sci1_rx_extract_idx %= SCI1_RX_BUFFER_SIZE; // limit the pointer
  return ch;
}


/* sci1 receive isr
 * direct sci1 receive error vector to here also
 */
#pragma interrupt
void sci1_rcv_isr(void)
{
  unsigned char status;
  unsigned short temp;

  status = SSR1;                        // get status register
  SSR1 = status & 0x87;                 // clear RDRF & error flags

  if (status & 0x40)                    // check RDRF bit
    {
    temp = (sci1_rx_insert_idx + 1) % SCI1_RX_BUFFER_SIZE;
    sci1_rx_buffer[sci1_rx_insert_idx] = RDR1; // read the character

    if (temp != sci1_rx_extract_idx)
      sci1_rx_insert_idx = temp;
    }
}


/* sci1 transmit isr
 */
#pragma interrupt
void sci1_xmt_isr(void)
{
  if (sci1_tx_insert_idx != sci1_tx_extract_idx)
    {
    TDR1 = sci1_tx_buffer[sci1_tx_extract_idx++];
    sci1_tx_extract_idx %= SCI1_TX_BUFFER_SIZE;

    /* clear this interrupt */
    asm(" bld  #7,@0xFFFFBC:8");        // read SSR1
    asm(" bist #7,@0xFFFFBC:8");        // clear TDRE
    }
  else
    {
    /* disable transmit interrupts */
    asm(" bclr #7,@0xFFFFBA:8");        // clear TIE in SCR1
    }
}

#endif
