/*;***********************************************************************
sci.h
  H8/300H SCI-0 & SCI-1 serial I/O routines

  Copyright 1999, R O SoftWare

*************************************************************************/

#ifndef INC_SCI_H
#define INC_SCI_H

/******************************************************
 * USER OPTIONS FOLLOW
 * Edit to match your configuration and requirements
 *****************************************************/
#define CPU_HZ        (15e6)            // processor speed in hertz
#define SCI0_SUPPORT  (1)               // non-zero to enable sci0 code
#define SCI1_SUPPORT  (0)               // non-zero to enable sci1 code

/* code is optimized for power of 2 buffer sizes (16, 32, 64, 128, ...) */
#define SCI0_RX_BUFFER_SIZE 512         // sci0 receive buffer size
#define SCI0_TX_BUFFER_SIZE 1024        // sci0 transmit buffer size
#define SCI1_RX_BUFFER_SIZE 256         // sci1 receive buffer size
#define SCI1_TX_BUFFER_SIZE 64          // sci1 transmit buffer size

/******************************************************
 * END OF USER OPTIONS AREA
 *****************************************************/

#ifndef BIT
#define BIT(n)    (1 << n)
#endif

/* MACRO for setting sci baud rate using sysclock divide by one.
 * Use with CONSTANTs for any baud rate from 2400 thru 38400 bps.
 * Good for CPU rates from 16MHz down to 4.9152MHz.  For other
 * combinations of CPU clock &/or baud rate, you will need to
 * adjust CKE-1..0 bits in the SCR (currently fixed at 0).
 * CAUTION: Passing a variable to this MACRO will include LOTS
 * of code from the floating point library.  USE CONSTANTs!
 * Magic formula - let compiler do the calculation & rounding!
 */
#define BAUD(n)   (unsigned char)((CPU_HZ / (32.0 * (n))) - .5)

/* Some predefined BAUD rates */
#define B2400     BAUD(2400)
#define B4800     BAUD(4800)
#define B9600     BAUD(9600)
#define B19200    BAUD(19200)
#define B38400    BAUD(38400)

/* Definitions for configuring BITs in SMR */
#define SMR_8BIT  (0)
#define SMR_7BIT  BIT(6)
#define SMR_NONE  (0)
#define SMR_EVEN  BIT(5)
#define SMR_ODD   BIT(5) + BIT(4)
#define SMR_1STOP (0)
#define SMR_2STOP BIT(3)

/* Definitions for typical SCI settings */
#define SCI_8N1   (unsigned char)(SMR_8BIT + SMR_NONE + SMR_1STOP)
#define SCI_7N1   (unsigned char)(SMR_7BIT + SMR_NONE + SMR_1STOP)
#define SCI_8N2   (unsigned char)(SMR_8BIT + SMR_NONE + SMR_2STOP)
#define SCI_7N2   (unsigned char)(SMR_7BIT + SMR_NONE + SMR_2STOP)
#define SCI_8E1   (unsigned char)(SMR_8BIT + SMR_EVEN + SMR_1STOP)
#define SCI_7E1   (unsigned char)(SMR_7BIT + SMR_EVEN + SMR_1STOP)
#define SCI_8E2   (unsigned char)(SMR_8BIT + SMR_EVEN + SMR_2STOP)
#define SCI_7E2   (unsigned char)(SMR_7BIT + SMR_EVEN + SMR_2STOP)
#define SCI_8O1   (unsigned char)(SMR_8BIT + SMR_ODD  + SMR_1STOP)
#define SCI_7O1   (unsigned char)(SMR_7BIT + SMR_ODD  + SMR_1STOP)
#define SCI_8O2   (unsigned char)(SMR_8BIT + SMR_ODD  + SMR_2STOP)
#define SCI_7O2   (unsigned char)(SMR_7BIT + SMR_ODD  + SMR_2STOP)


#if SCI0_SUPPORT

/* Initialize sci0 for async mode
 * typical call - sci0_init(BAUD(9600), SCI_8N1);
 *           or - sci0_init(B9600, SCI_8N1);
 */
void sci0_init(unsigned char baud, unsigned char config);

/* Send a character via a transmit channel.
 * Return 0 on success, -1 on error (queue full).
 */
int sci0_putc(char ch);

/* get available space in sci transmit queue
 */
unsigned short sci0_space(void);

/* write NULL terminated 'string' to the sci output
 * queue, returning a pointer to the next character
 * to be written.  ('\0' if full string is written)
 */
char *sci0_puts(char *string);

/* Write 'count' characters from 'buffer' to the sci
 * output queue.
 * Return 0 on success, -1 if insufficient room, -2 on error
 * NOTE: if insufficient room, no characters are written.
 */
int sci0_write(char *buffer, unsigned short count);

/* Get a character via receive channel.
 * Return character on success, -1 if no character is available.
 */
int sci0_getc(void);

/* sci0 receive isr
 * direct sci0 receive error vector to here also
 */
void sci0_rcv_isr(void);

/* sci0 transmit isr
 */
void sci0_xmt_isr(void);

#endif


#if SCI1_SUPPORT

/* Initialize sci1 for async mode
 * typical call - sci1_init(BAUD(9600), SCI_8N1);
 *           or - sci1_init(B9600, SCI_8N1);
 */
void sci1_init(unsigned char baud, unsigned char config);

/* Send a character via a transmit channel.
 * Return 0 on success, -1 on error (queue full).
 */
int sci1_putc(char ch);

/* get available space in sci transmit queue
 */
unsigned short sci1_space(void);

/* write NULL terminated 'string' to the sci output
 * queue, returning a pointer to the next character
 * to be written.  ('\0' if full string is written)
 */
char *sci1_puts(char *string);

/* Write 'count' characters from 'buffer' to the sci
 * output queue.
 * Return 0 on success, -1 if insufficient room, -2 on error
 * NOTE: if insufficient room, no characters are written.
 */
int sci1_write(char *buffer, unsigned short count);

/* Get a character via receive channel.
 * Return character on success, -1 if no character is available.
 */
int sci1_getc(void);

/* sci1 receive isr
 * direct sci1 receive error vector to here also
 */
void sci1_rcv_isr(void);

/* sci1 transmit isr
 */
void sci1_xmt_isr(void);

#endif

#endif
